@@ -2,51 +2,66 @@
 
 # devstack plugin for octavia
 
+GET_PIP_CACHE_LOCATION=/opt/stack/cache/files/get-pip.py
+
 function octavia_install {
 
     setup_develop $OCTAVIA_DIR
-    if ! [ "$DISABLE_AMP_IMAGE_BUILD" == 'True' ]; then
-        install_package qemu kpartx
-        git_clone $TRIPLEO_IMAGE_ELEMENTS_REPO $TRIPLEO_IMAGE_ELEMENTS_DIR $TRIPLEO_IMAGE_ELEMENTS_BRANCH
+    if [ $OCTAVIA_NODE == 'main' ] || [ $OCTAVIA_NODE == 'standalone' ] ; then
+        if ! [ "$DISABLE_AMP_IMAGE_BUILD" == 'True' ]; then
+            install_package qemu kpartx
+            git_clone $DISKIMAGE_BUILDER_REPO $DISKIMAGE_BUILDER_DIR $DISKIMAGE_BUILDER_BRANCH
+            sudo -H -E pip install -r $DEST/diskimage-builder/requirements.txt
+        fi
     fi
 }
 
+function set_octavia_worker_image_owner_id {
+    image_id=$(openstack image list --property name=${OCTAVIA_AMP_IMAGE_NAME} -f value -c ID)
+    owner_id=$(openstack image show ${image_id} -c owner -f value)
+    iniset $OCTAVIA_CONF controller_worker amp_image_owner_id ${owner_id}
+}
+
 function build_octavia_worker_image {
 
     # pull the agent code from the current code zuul has a reference to
+    if [ -n "$DIB_REPOLOCATION_pip_and_virtualenv" ]; then
+        export DIB_REPOLOCATION_pip_and_virtualenv=$DIB_REPOLOCATION_pip_and_virtualenv
+    elif [ -f $GET_PIP_CACHE_LOCATION ] ; then
+        export DIB_REPOLOCATION_pip_and_virtualenv=file://$GET_PIP_CACHE_LOCATION
+    fi
     export DIB_REPOLOCATION_amphora_agent=$OCTAVIA_DIR
     export DIB_REPOREF_amphora_agent=$(git -C "$OCTAVIA_DIR" log -1 --pretty="format:%H")
     TOKEN=$(openstack token issue | grep ' id ' | get_field 2)
     die_if_not_set $LINENO TOKEN "Keystone failed to get token."
 
+    octavia_dib_tracing_arg=
+    if [ "$OCTAVIA_DIB_TRACING" != "0" ]; then
+        octavia_dib_tracing_arg="-x"
+    fi
     if ! [ -f $OCTAVIA_AMP_IMAGE_FILE ]; then
-        $OCTAVIA_DIR/diskimage-create/diskimage-create.sh -s 2 -o $OCTAVIA_AMP_IMAGE_FILE
+        $OCTAVIA_DIR/diskimage-create/diskimage-create.sh $octavia_dib_tracing_arg -s 2 -o $OCTAVIA_AMP_IMAGE_FILE
     fi
     upload_image file://${OCTAVIA_AMP_IMAGE_FILE} $TOKEN
 
-    image_id=$(glance image-list --property-filter name=${OCTAVIA_AMP_IMAGE_NAME} | awk '/ amphora-x64-haproxy / {print $2}')
-    owner_id=$(glance image-show ${image_id} | awk '/ owner / {print $4}')
-    iniset $OCTAVIA_CONF controller_worker amp_image_owner_id ${owner_id}
-
 }
 
 function create_octavia_accounts {
-        create_service_user "neutron"
+    create_service_user "octavia"
 
-        if [[ "$KEYSTONE_CATALOG_BACKEND" = 'sql' ]]; then
-
-            local neutron_service=$(get_or_create_service "octavia" \
-                "octavia" "Octavia Service")
-            get_or_create_endpoint $neutron_service \
-                "$REGION_NAME" \
-                "$OCTAVIA_PROTOCOL://$SERVICE_HOST:$OCTAVIA_PORT/" \
-                "$OCTAVIA_PROTOCOL://$SERVICE_HOST:$OCTAVIA_PORT/" \
-                "$OCTAVIA_PROTOCOL://$SERVICE_HOST:$OCTAVIA_PORT/"
-        fi
+    local octavia_service=$(get_or_create_service "octavia" \
+        "octavia" "Octavia Service")
+    get_or_create_endpoint $octavia_service \
+        "$REGION_NAME" \
+        "$OCTAVIA_PROTOCOL://$SERVICE_HOST:$OCTAVIA_PORT/" \
+        "$OCTAVIA_PROTOCOL://$SERVICE_HOST:$OCTAVIA_PORT/" \
+        "$OCTAVIA_PROTOCOL://$SERVICE_HOST:$OCTAVIA_PORT/"
 }
 
 function octavia_configure {
 
+    create_octavia_cache_dir
+
     sudo mkdir -m 755 -p $OCTAVIA_CONF_DIR
     safe_chown $STACK_USER $OCTAVIA_CONF_DIR
 
@@ -59,18 +74,20 @@ function octavia_configure {
 
     iniset $OCTAVIA_CONF database connection "mysql+pymysql://${DATABASE_USER}:${DATABASE_PASSWORD}@${DATABASE_HOST}:3306/octavia"
 
-    if [ "$OCTAVIA_AUTH_VERSION" == "2" ] ; then
-        AUTH_URI=${KEYSTONE_AUTH_URI%/v3}/v2.0
-    else
-        AUTH_URI=${KEYSTONE_AUTH_URI}
-    fi
-    iniset $OCTAVIA_CONF keystone_authtoken auth_uri ${AUTH_URI}
-    iniset $OCTAVIA_CONF keystone_authtoken admin_user ${OCTAVIA_ADMIN_USER}
-    iniset $OCTAVIA_CONF keystone_authtoken admin_tenant_name ${OCTAVIA_ADMIN_TENANT_NAME}
-    iniset $OCTAVIA_CONF keystone_authtoken admin_password ${OCTAVIA_ADMIN_PASSWORD}
-    iniset $OCTAVIA_CONF keystone_authtoken auth_version ${OCTAVIA_AUTH_VERSION}
+    # Configure keystone auth_token for all users
+    configure_auth_token_middleware $OCTAVIA_CONF octavia $OCTAVIA_AUTH_CACHE_DIR
 
-    iniset $OCTAVIA_CONF controller_worker amp_flavor_id ${OCTAVIA_AMP_FLAVOR_ID}
+    # Ensure config is set up properly for authentication as admin
+    iniset $OCTAVIA_CONF service_auth auth_url $KEYSTONE_AUTH_URI
+    iniset $OCTAVIA_CONF service_auth auth_type password
+    iniset $OCTAVIA_CONF service_auth username $OCTAVIA_USERNAME
+    iniset $OCTAVIA_CONF service_auth password $OCTAVIA_PASSWORD
+    iniset $OCTAVIA_CONF service_auth user_domain_name $OCTAVIA_USER_DOMAIN_NAME
+    iniset $OCTAVIA_CONF service_auth project_name $OCTAVIA_PROJECT_NAME
+    iniset $OCTAVIA_CONF service_auth project_domain_name $OCTAVIA_PROJECT_DOMAIN_NAME
+    iniset $OCTAVIA_CONF service_auth cafile $SSL_BUNDLE_FILE
+    iniset $OCTAVIA_CONF service_auth signing_dir $signing_dir
+    iniset $OCTAVIA_CONF service_auth memcached_servers $SERVICE_HOST:11211
 
     # Setting other required default options
     iniset $OCTAVIA_CONF controller_worker amphora_driver ${OCTAVIA_AMPHORA_DRIVER}
@@ -99,12 +116,18 @@ function octavia_configure {
     iniuncomment $OCTAVIA_CONF haproxy_amphora base_cert_dir
     iniuncomment $OCTAVIA_CONF haproxy_amphora connection_max_retries
     iniuncomment $OCTAVIA_CONF haproxy_amphora connection_retry_interval
+    iniuncomment $OCTAVIA_CONF haproxy_amphora rest_request_conn_timeout
+    iniuncomment $OCTAVIA_CONF haproxy_amphora rest_request_read_timeout
+    iniuncomment $OCTAVIA_CONF controller_worker amp_active_retries
+    iniuncomment $OCTAVIA_CONF controller_worker amp_active_wait_sec
 
     # devstack optimizations for tempest runs
     iniset $OCTAVIA_CONF haproxy_amphora connection_max_retries 1500
     iniset $OCTAVIA_CONF haproxy_amphora connection_retry_interval 1
+    iniset $OCTAVIA_CONF haproxy_amphora rest_request_conn_timeout ${OCTAVIA_AMP_CONN_TIMEOUT}
+    iniset $OCTAVIA_CONF haproxy_amphora rest_request_read_timeout ${OCTAVIA_AMP_READ_TIMEOUT}
     iniset $OCTAVIA_CONF controller_worker amp_active_retries 100
-    iniset $OCTAVIA_CONF controller_worker amp_active_wait_sec 1
+    iniset $OCTAVIA_CONF controller_worker amp_active_wait_sec 2
 
     if [[ -a $OCTAVIA_SSH_DIR ]] ; then
         rm -rf $OCTAVIA_SSH_DIR
@@ -152,17 +175,30 @@ function octavia_configure {
 }
 
 function create_mgmt_network_interface {
-    # Create security group and rules
-    neutron security-group-create lb-health-mgr-sec-grp
-    neutron security-group-rule-create --protocol udp --port-range-min $OCTAVIA_HM_LISTEN_PORT --port-range-max $OCTAVIA_HM_LISTEN_PORT lb-health-mgr-sec-grp
+    if [ $OCTAVIA_MGMT_PORT_IP != 'auto' ]; then
+        SUBNET_ID=$(neutron subnet-show lb-mgmt-subnet | awk '/ id / {print $4}')
+        PORT_FIXED_IP="--fixed-ip subnet_id=$SUBNET_ID,ip_address=$OCTAVIA_MGMT_PORT_IP"
+    fi
 
-    id_and_mac=$(neutron port-create --name octavia-health-manager-$OCTAVIA_NODE-listen-port --security-group lb-health-mgr-sec-grp --device-owner Octavia:health-mgr --binding:host_id=$(hostname) lb-mgmt-net | awk '/ id | mac_address / {print $4}')
+    # TODO(johnsom) Change this to OSC when security group is working
+    id_and_mac=$(neutron port-create --name octavia-health-manager-$OCTAVIA_NODE-listen-port --security-group lb-health-mgr-sec-grp --device-owner Octavia:health-mgr --binding:host_id=$(hostname) lb-mgmt-net $PORT_FIXED_IP | awk '/ id | mac_address / {print $4}')
 
     id_and_mac=($id_and_mac)
     MGMT_PORT_ID=${id_and_mac[0]}
     MGMT_PORT_MAC=${id_and_mac[1]}
-    MGMT_PORT_IP=$(neutron port-show $MGMT_PORT_ID | awk '/ "ip_address": / {print $7; exit}' | sed -e 's/"//g' -e 's/,//g' -e 's/}//g')
-    sudo ovs-vsctl -- --may-exist add-port ${OVS_BRIDGE:-br-int} o-hm0 -- set Interface o-hm0 type=internal -- set Interface o-hm0 external-ids:iface-status=active -- set Interface o-hm0 external-ids:attached-mac=$MGMT_PORT_MAC -- set Interface o-hm0 external-ids:iface-id=$MGMT_PORT_ID
+    # TODO(johnsom) This gets the IPv4 address, should be updated for IPv6
+    MGMT_PORT_IP=$(openstack port show -f value -c fixed_ips $MGMT_PORT_ID | awk '{FS=",| "; gsub(",",""); gsub("'\''",""); for(i = 1; i <= NF; ++i) {if ($i ~ /^ip_address/) {n=index($i, "="); if (substr($i, n+1) ~ "\\.") print substr($i, n+1)}}}')
+    if [[ $Q_AGENT == "openvswitch" ]]; then
+        sudo ovs-vsctl -- --may-exist add-port ${OVS_BRIDGE:-br-int} o-hm0 -- set Interface o-hm0 type=internal -- set Interface o-hm0 external-ids:iface-status=active -- set Interface o-hm0 external-ids:attached-mac=$MGMT_PORT_MAC -- set Interface o-hm0 external-ids:iface-id=$MGMT_PORT_ID
+    elif [[ $Q_AGENT == "linuxbridge" ]]; then
+        if ! ip link show o-hm0 ; then
+            sudo ip link add o-hm0 type veth peer name o-bhm0
+            NETID=$(openstack network show lb-mgmt-net -c id -f value)
+            BRNAME=brq$(echo $NETID|cut -c 1-11)
+            sudo brctl addif $BRNAME o-bhm0
+            sudo ip link set o-bhm0 up
+        fi
+    fi
     sudo ip link set dev o-hm0 address $MGMT_PORT_MAC
     sudo dhclient -v o-hm0 -cf $OCTAVIA_DHCLIENT_CONF
     sudo iptables -I INPUT -i o-hm0 -p udp --dport 5555 -j ACCEPT
@@ -175,22 +211,32 @@ function create_mgmt_network_interface {
 
     iniset $OCTAVIA_CONF health_manager bind_ip $MGMT_PORT_IP
     iniset $OCTAVIA_CONF health_manager bind_port $OCTAVIA_HM_LISTEN_PORT
+
 }
 
 function build_mgmt_network {
     # Create network and attach a subnet
-    OCTAVIA_AMP_NETWORK_ID=$(neutron net-create lb-mgmt-net | awk '/ id / {print $4}')
-    OCTAVIA_AMP_SUBNET_ID=$(neutron subnet-create --name lb-mgmt-subnet --allocation-pool start=$OCTAVIA_MGMT_SUBNET_START,end=$OCTAVIA_MGMT_SUBNET_END lb-mgmt-net $OCTAVIA_MGMT_SUBNET | awk '/ id / {print $4}')
+    OCTAVIA_AMP_NETWORK_ID=$(openstack network create lb-mgmt-net | awk '/ id / {print $4}')
+    OCTAVIA_AMP_SUBNET_ID=$(openstack subnet create --subnet-range $OCTAVIA_MGMT_SUBNET --allocation-pool start=$OCTAVIA_MGMT_SUBNET_START,end=$OCTAVIA_MGMT_SUBNET_END --network lb-mgmt-net lb-mgmt-subnet | awk '/ id / {print $4}')
 
     # Create security group and rules
-    neutron security-group-create lb-mgmt-sec-grp
-    neutron security-group-rule-create --protocol icmp lb-mgmt-sec-grp
-    neutron security-group-rule-create --protocol tcp --port-range-min 22 --port-range-max 22 lb-mgmt-sec-grp
-    neutron security-group-rule-create --protocol tcp --port-range-min 9443 --port-range-max 9443 lb-mgmt-sec-grp
+    openstack security group create lb-mgmt-sec-grp
+    openstack security group rule create --protocol icmp lb-mgmt-sec-grp
+    openstack security group rule create --protocol tcp --dst-port 22 lb-mgmt-sec-grp
+    openstack security group rule create --protocol tcp --dst-port 9443 lb-mgmt-sec-grp
+    openstack security group rule create --protocol icmpv6 --ethertype IPv6 --remote-ip ::/0 lb-mgmt-sec-grp
+    openstack security group rule create --protocol tcp --dst-port 22 --ethertype IPv6 --remote-ip ::/0 lb-mgmt-sec-grp
+    openstack security group rule create --protocol tcp --dst-port 9443 --ethertype IPv6 --remote-ip ::/0 lb-mgmt-sec-grp
 
-    OCTAVIA_MGMT_SEC_GRP_ID=$(nova secgroup-list | awk ' / lb-mgmt-sec-grp / {print $2}')
+    # Create security group and rules
+    openstack security group create lb-health-mgr-sec-grp
+    openstack security group rule create --protocol udp --dst-port $OCTAVIA_HM_LISTEN_PORT lb-health-mgr-sec-grp
+    openstack security group rule create --protocol udp --dst-port $OCTAVIA_HM_LISTEN_PORT --ethertype IPv6 --remote-ip ::/0 lb-health-mgr-sec-grp
+}
+
+function configure_lb_mgmt_sec_grp {
+    OCTAVIA_MGMT_SEC_GRP_ID=$(openstack security group list | awk ' / lb-mgmt-sec-grp / {print $2}')
     iniset ${OCTAVIA_CONF} controller_worker amp_secgroup_list ${OCTAVIA_MGMT_SEC_GRP_ID}
-
 }
 
 function configure_octavia_tempest {
@@ -204,17 +250,20 @@ function configure_octavia_tempest {
 }
 
 function create_amphora_flavor {
-    nova flavor-create --is-public False m1.amphora ${OCTAVIA_AMP_FLAVOR_ID} 1024 2 1
+    # Pass even if it exists to avoid race condition on multinode
+    openstack flavor create --id auto --ram 1024 --disk 2 --vcpus 1 --private m1.amphora -f value -c id || true
+    amp_flavor_id=$(openstack flavor list --all -c ID -c Name | awk ' / m1.amphora / {print $2}')
+    iniset $OCTAVIA_CONF controller_worker amp_flavor_id $amp_flavor_id
 }
 
 function configure_octavia_api_haproxy {
 
+    install_package haproxy
+
     cp ${OCTAVIA_DIR}/devstack/etc/octavia/haproxy.cfg ${OCTAVIA_CONF_DIR}/haproxy.cfg
 
     sed -i.bak "s/OCTAVIA_PORT/${OCTAVIA_PORT}/" ${OCTAVIA_CONF_DIR}/haproxy.cfg
 
-    iniset $OCTAVIA_CONF DEFAULT bind_port ${OCTAVIA_HA_PORT}
-
     NODES=(${OCTAVIA_NODES//,/ })
 
     for NODE in ${NODES[@]}; do
@@ -240,18 +289,27 @@ function octavia_start {
 
     if [ $OCTAVIA_NODE == 'main' ] || [ $OCTAVIA_NODE == 'standalone' ] ; then
         # things that should only happen on the ha main node / or once
-        nova keypair-add --pub-key ${OCTAVIA_AMP_SSH_KEY_PATH}.pub ${OCTAVIA_AMP_SSH_KEY_NAME}
+        openstack keypair create --public-key ${OCTAVIA_AMP_SSH_KEY_PATH}.pub ${OCTAVIA_AMP_SSH_KEY_NAME}
+
+        # Check if an amphora image is already loaded
+        AMPHORA_IMAGE_NAME=$(openstack image list --property name=${OCTAVIA_AMP_IMAGE_NAME} -f value -c Name)
+        export AMPHORA_IMAGE_NAME
+
+        if [ "$AMPHORA_IMAGE_NAME" == ${OCTAVIA_AMP_IMAGE_NAME} ]; then
+            echo "Found existing amphora image: $AMPHORA_IMAGE_NAME"
+            echo "Skipping amphora image build"
+            export DISABLE_AMP_IMAGE_BUILD=True
+        fi
 
         if ! [ "$DISABLE_AMP_IMAGE_BUILD" == 'True' ]; then
             build_octavia_worker_image
         fi
 
-        OCTAVIA_AMP_IMAGE_ID=$(glance image-list | grep ${OCTAVIA_AMP_IMAGE_NAME} | awk '{print $2}')
+        OCTAVIA_AMP_IMAGE_ID=$(openstack image list -f value --property name=${OCTAVIA_AMP_IMAGE_NAME} -c ID)
 
         if [ -n "$OCTAVIA_AMP_IMAGE_ID" ]; then
-            glance image-tag-update ${OCTAVIA_AMP_IMAGE_ID} ${OCTAVIA_AMP_IMAGE_TAG}
+            openstack image set --tag ${OCTAVIA_AMP_IMAGE_TAG} ${OCTAVIA_AMP_IMAGE_ID}
         fi
-        create_amphora_flavor
 
         # Create a management network.
         build_mgmt_network
@@ -262,21 +320,33 @@ function octavia_start {
         if is_service_enabled tempest; then
             configure_octavia_tempest ${OCTAVIA_AMP_NETWORK_ID}
         fi
-    else
-        OCTAVIA_AMP_IMAGE_ID=$(glance image-list | grep ${OCTAVIA_AMP_IMAGE_NAME} | awk '{print $2}')
     fi
 
+    if ! [ "$DISABLE_AMP_IMAGE_BUILD" == 'True' ]; then
+        set_octavia_worker_image_owner_id
+    fi
+    create_amphora_flavor
+
     create_mgmt_network_interface
+    configure_lb_mgmt_sec_grp
 
     iniset $OCTAVIA_CONF controller_worker amp_image_tag ${OCTAVIA_AMP_IMAGE_TAG}
 
-    OCTAVIA_AMP_NETWORK_ID=$(neutron net-list | awk '/ lb-mgmt-net / {print $2}')
+    OCTAVIA_AMP_NETWORK_ID=$(openstack network list | awk '/ lb-mgmt-net / {print $2}')
 
     iniset $OCTAVIA_CONF controller_worker amp_boot_network_list ${OCTAVIA_AMP_NETWORK_ID}
 
     if [ $OCTAVIA_NODE == 'main' ]; then
         configure_octavia_api_haproxy
         run_process $OCTAVIA_API_HAPROXY "/usr/sbin/haproxy -db -V -f ${OCTAVIA_CONF_DIR}/haproxy.cfg"
+        # make sure octavia is reachable from haproxy
+        iniset $OCTAVIA_CONF DEFAULT bind_port ${OCTAVIA_HA_PORT}
+        iniset $OCTAVIA_CONF DEFAULT bind_host 0.0.0.0
+    fi
+    if [ $OCTAVIA_NODE != 'main' ] && [ $OCTAVIA_NODE != 'standalone' ] ; then
+        # make sure octavia is reachable from haproxy from main node
+        iniset $OCTAVIA_CONF DEFAULT bind_port ${OCTAVIA_HA_PORT}
+        iniset $OCTAVIA_CONF DEFAULT bind_host 0.0.0.0
     fi
 
     run_process $OCTAVIA_API  "$OCTAVIA_API_BINARY $OCTAVIA_API_ARGS"
@@ -292,13 +362,20 @@ function octavia_stop {
     # Kill dhclient process started for o-hm0 interface
     pids=$(ps aux | awk '/o-hm0/ { print $2 }')
     [ ! -z "$pids" ] && sudo kill $pids
+    if [[ $Q_AGENT == "linuxbridge" ]]; then
+        if ip link show o-hm0 ; then
+            sudo ip link del o-hm0
+        fi
+    fi
 }
+
 function octavia_configure_common {
     if is_service_enabled $OCTAVIA_SERVICE && [ $OCTAVIA_NODE == 'main' ] || [ $OCTAVIA_NODE = 'standalone' ] ; then
         inicomment $NEUTRON_LBAAS_CONF service_providers service_provider
         iniadd $NEUTRON_LBAAS_CONF service_providers service_provider $OCTAVIA_SERVICE_PROVIDER
     fi
 }
+
 function octavia_cleanup {
 
     if [ ${OCTAVIA_AMP_IMAGE_NAME}x != x ] ; then
@@ -318,9 +395,18 @@ function octavia_cleanup {
     fi
     if [ $OCTAVIA_NODE == 'main' ] || [ $OCTAVIA_NODE == 'standalone' ] ; then
         if [ ${OCTAVIA_AMP_SSH_KEY_NAME}x != x ] ; then
-            nova keypair-delete ${OCTAVIA_AMP_SSH_KEY_NAME}
+            openstack keypair delete ${OCTAVIA_AMP_SSH_KEY_NAME}
         fi
     fi
+
+    sudo rm -rf $NOVA_STATE_PATH $NOVA_AUTH_CACHE_DIR
+}
+
+# create_octavia_cache_dir() - Part of the configure_octavia() process
+function create_octavia_cache_dir {
+    # Create cache dir
+    sudo install -d -o $STACK_USER $OCTAVIA_AUTH_CACHE_DIR
+    rm -f $OCTAVIA_AUTH_CACHE_DIR/*
 }
 
 # check for service enabled
@@ -330,21 +416,11 @@ if is_service_enabled $OCTAVIA; then
             die "The neutron $Q_SVC service must be enabled to use $OCTAVIA"
         fi
 
-        # Check if an amphora image is already loaded
-        AMPHORA_IMAGE_NAME=$(glance image-list | awk '/ amphora-x64-haproxy / {print $4}')
-        export AMPHORA_IMAGE_NAME
-
         if [ "$DISABLE_AMP_IMAGE_BUILD" == 'True' ]; then
             echo "Found DISABLE_AMP_IMAGE_BUILD == True"
             echo "Skipping amphora image build"
         fi
 
-        if [ "$AMPHORA_IMAGE_NAME" == 'amphora-x64-haproxy' ]; then
-            echo "Found existing amphora image: $AMPHORA_IMAGE_NAME"
-            echo "Skipping amphora image build"
-            DISABLE_AMP_IMAGE_BUILD=True
-            export DISABLE_AMP_IMAGE_BUILD
-        fi
     fi
 
     if [[ "$1" == "stack" && "$2" == "install" ]]; then
