@@ -23,7 +23,7 @@ usage() {
     echo "            [-a i386 | **amd64** | armhf ]"
     echo "            [-b **haproxy** ]"
     echo "            [-c **~/.cache/image-create** | <cache directory> ]"
-    echo "            [-d **trusty** | xenial | <other release id> ]"
+    echo "            [-d **xenial** | trusty | <other release id> ]"
     echo "            [-h]"
     echo "            [-i **ubuntu** | fedora | centos | rhel ]"
     echo "            [-o **amphora-x64-haproxy** | <filename> ]"
@@ -36,7 +36,7 @@ usage() {
     echo "        '-a' is the architecture type for the image (default: amd64)"
     echo "        '-b' is the backend type (default: haproxy)"
     echo "        '-c' is the path to the cache directory (default: ~/.cache/image-create)"
-    echo "        '-d' distribution release id (default: trusty)"
+    echo "        '-d' distribution release id (default on ubuntu: xenial)"
     echo "        '-h' display this help message"
     echo "        '-i' is the base OS (default: ubuntu)"
     echo "        '-o' is the output image file name"
@@ -45,6 +45,7 @@ usage() {
     echo "        '-t' is the image type (default: qcow2)"
     echo "        '-v' display the script version"
     echo "        '-w' working directory for image building (default: .)"
+    echo "        '-x' enable tracing for diskimage-builder"
     echo
     exit 1
 }
@@ -71,8 +72,9 @@ if [ -z $OCTAVIA_REPO_PATH ]; then
     AMP_DIR=$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )
     OCTAVIA_REPO_PATH=${OCTAVIA_REPO_PATH:-${AMP_DIR%/*}}
 fi
+dib_enable_tracing=
 
-while getopts "a:b:c:d:hi:o:t:r:s:vw:" opt; do
+while getopts "a:b:c:d:hi:o:t:r:s:vw:x" opt; do
     case $opt in
         a)
             AMP_ARCH=$OPTARG
@@ -138,6 +140,8 @@ while getopts "a:b:c:d:hi:o:t:r:s:vw:" opt; do
         w)
             AMP_WORKING_DIR=$OPTARG
         ;;
+        x)  dib_enable_tracing=1
+        ;;
         *)
             usage
         ;;
@@ -158,7 +162,11 @@ AMP_CACHEDIR=${AMP_CACHEDIR:-"$HOME/.cache/image-create"}
 
 AMP_BASEOS=${AMP_BASEOS:-"ubuntu"}
 
-export DIB_RELEASE=${AMP_DIB_RELEASE:-"trusty"}
+if [ "$AMP_BASEOS" = "ubuntu" ]; then
+    export DIB_RELEASE=${AMP_DIB_RELEASE:-"xenial"}
+else
+    export DIB_RELEASE=${AMP_DIB_RELEASE}
+fi
 
 AMP_OUTPUTFILENAME=${AMP_OUTPUTFILENAME:-"$PWD/amphora-x64-haproxy"}
 
@@ -191,21 +199,10 @@ fi
 # For system-wide installs, DIB will automatically find the elements, so we only check local path
 DIB_ELEMENTS_PATH=${DIB_REPO_PATH:-${OCTAVIA_REPO_PATH%/*}/diskimage-builder/elements}
 
-TRIPLEO_ELEMENTS_PATH=${TRIPLEO_ELEMENTS_PATH:-${OCTAVIA_REPO_PATH%/*}/tripleo-image-elements/elements}
-
-if ! [ -d $TRIPLEO_ELEMENTS_PATH ]; then
-    SYSTEM_TRIPLEO_ELEMENTS_PATH=$(find_system_elements tripleo-image-elements)
-    if [ -z ${SYSTEM_TRIPLEO_ELEMENTS_PATH} ]; then
-        echo "ERROR: tripleo-image-elements elements directory not found at: " $TRIPLEO_ELEMENTS_PATH " Exiting."
-        exit 1
-    fi
-    TRIPLEO_ELEMENTS_PATH=${SYSTEM_TRIPLEO_ELEMENTS_PATH}
-fi
-
 if [ "$DIB_LOCAL_ELEMENTS_PATH" ]; then
-    export ELEMENTS_PATH=$DIB_ELEMENTS_PATH:$TRIPLEO_ELEMENTS_PATH:$OCTAVIA_ELEMENTS_PATH:$DIB_LOCAL_ELEMENTS_PATH
+    export ELEMENTS_PATH=$DIB_ELEMENTS_PATH:$OCTAVIA_ELEMENTS_PATH:$DIB_LOCAL_ELEMENTS_PATH
 else
-    export ELEMENTS_PATH=$DIB_ELEMENTS_PATH:$TRIPLEO_ELEMENTS_PATH:$OCTAVIA_ELEMENTS_PATH
+    export ELEMENTS_PATH=$DIB_ELEMENTS_PATH:$OCTAVIA_ELEMENTS_PATH
 fi
 
 export CLOUD_INIT_DATASOURCES=${CLOUD_INIT_DATASOURCES:-"ConfigDrive"}
@@ -270,13 +267,23 @@ elif [ "$platform" = 'NAME=Fedora' ]; then
     done
 else
     # centos or rhel
-        PKG_LIST="qemu-kvm qemu-img kpartx git"
+        PKG_LIST="qemu-kvm qemu-img"
+        for pkg in $PKG_LIST; do
+            if ! yum list installed $pkg &> /dev/null; then
+                if ! yum list installed $pkg"-ev" &> /dev/null; then
+                    echo "Required package " $pkg " or " $pkg"-ev" " is not installed.  Exiting."
+                    exit 1
+                fi
+            fi
+        done
+        PKG_LIST="kpartx git"
         for pkg in $PKG_LIST; do
             if ! yum list installed $pkg &> /dev/null; then
                 echo "Required package " $pkg " is not installed.  Exiting."
                 exit 1
             fi
         done
+
         if [ ${platform:0:6} = "CentOS" ]; then
             # install EPEL repo, in order to install argparse
             PKG_LIST="python-argparse"
@@ -326,21 +333,21 @@ if [ "$AMP_BASEOS" = "ubuntu" ]; then
         export UBUNTU_MIRROR="$BASE_OS_MIRROR"
     fi
 elif [ "$AMP_BASEOS" = "fedora" ]; then
-    AMP_element_sequence=${AMP_element_sequence:-"base vm fedora"}
+    AMP_element_sequence=${AMP_element_sequence:-"base vm fedora selinux-permissive"}
     AMP_element_sequence="$AMP_element_sequence $AMP_BACKEND"
     if [ "$BASE_OS_MIRROR" ]; then
         AMP_element_sequence="$AMP_element_sequence fedora-mirror"
         export FEDORA_MIRROR="$BASE_OS_MIRROR"
     fi
 elif [ "$AMP_BASEOS" = "centos" ]; then
-    AMP_element_sequence=${AMP_element_sequence:-"base vm centos7"}
+    AMP_element_sequence=${AMP_element_sequence:-"base vm centos7 epel selinux-permissive"}
     AMP_element_sequence="$AMP_element_sequence $AMP_BACKEND"
     if [ "$BASE_OS_MIRROR" ]; then
         AMP_element_sequence="$AMP_element_sequence centos-mirror"
         export CENTOS_MIRROR="$BASE_OS_MIRROR"
     fi
 elif [ "$AMP_BASEOS" = "rhel" ]; then
-    AMP_element_sequence=${AMP_element_sequence:-"base vm rhel7"}
+    AMP_element_sequence=${AMP_element_sequence:-"base vm rhel7 selinux-permissive"}
     AMP_element_sequence="$AMP_element_sequence $AMP_BACKEND"
 fi
 
@@ -382,7 +389,12 @@ if [ "$AMP_BASEOS" = "ubuntu" ]; then
     export DIB_CLOUD_INIT_DATASOURCES=$CLOUD_INIT_DATASOURCES
 fi
 
-disk-image-create -a $AMP_ARCH -o $AMP_OUTPUTFILENAME -t $AMP_IMAGETYPE --image-size $AMP_IMAGESIZE --image-cache $AMP_CACHEDIR $AMP_element_sequence
+dib_trace_arg=
+if [ -n "$dib_enable_tracing" ]; then
+    dib_trace_arg="-x"
+fi
+
+disk-image-create $dib_trace_arg -a $AMP_ARCH -o $AMP_OUTPUTFILENAME -t $AMP_IMAGETYPE --image-size $AMP_IMAGESIZE --image-cache $AMP_CACHEDIR $AMP_element_sequence
 
 popd > /dev/null # out of $TEMP
 rm -rf $TEMP
