@@ -28,18 +28,18 @@ from octavia.amphorae.drivers import driver_base as driver_base
 from octavia.amphorae.drivers.haproxy import exceptions as exc
 from octavia.amphorae.drivers.keepalived import vrrp_rest_driver
 from octavia.common.config import cfg
-from octavia.common import constants
+from octavia.common import constants as consts
 from octavia.common.jinja.haproxy import jinja_cfg
 from octavia.common.tls_utils import cert_parser
-from octavia.i18n import _LW
+from octavia.common import utils
+from octavia.i18n import _LE, _LW
 
 LOG = logging.getLogger(__name__)
-API_VERSION = constants.API_VERSION
+API_VERSION = consts.API_VERSION
 OCTAVIA_API_CLIENT = (
     "Octavia HaProxy Rest Client/{version} "
     "(https://wiki.openstack.org/wiki/Octavia)").format(version=API_VERSION)
 CONF = cfg.CONF
-CONF.import_group('haproxy_amphora', 'octavia.common.config')
 
 
 class HaproxyAmphoraLoadBalancerDriver(
@@ -66,22 +66,17 @@ class HaproxyAmphoraLoadBalancerDriver(
 
         # Process listener certificate info
         certs = self._process_tls_certificates(listener)
-        # Generate HaProxy configuration from listener object
-        config = self.jinja.build_config(listener, certs['tls_cert'],
-                                         certs['sni_certs'])
 
         for amp in listener.load_balancer.amphorae:
-            if amp.status != constants.DELETED:
+            if amp.status != consts.DELETED:
+                # Generate HaProxy configuration from listener object
+                config = self.jinja.build_config(
+                    host_amphora=amp,
+                    listener=listener,
+                    tls_cert=certs['tls_cert'],
+                    user_group=CONF.haproxy_amphora.user_group)
                 self.client.upload_config(amp, listener.id, config)
-                # todo (german): add a method to REST interface to reload or
-                #                start without having to check
-                # Is that listener running?
-                r = self.client.get_listener_status(amp,
-                                                    listener.id)
-                if r['status'] == 'ACTIVE':
-                    self.client.reload_listener(amp, listener.id)
-                else:
-                    self.client.start_listener(amp, listener.id)
+                self.client.reload_listener(amp, listener.id)
 
     def upload_cert_amp(self, amp, pem):
         LOG.debug("Amphora %s updating cert in REST driver "
@@ -91,7 +86,7 @@ class HaproxyAmphoraLoadBalancerDriver(
 
     def _apply(self, func, listener=None, *args):
         for amp in listener.load_balancer.amphorae:
-            if amp.status != constants.DELETED:
+            if amp.status != consts.DELETED:
                 func(amp, listener.id, *args)
 
     def stop(self, listener, vip):
@@ -112,27 +107,57 @@ class HaproxyAmphoraLoadBalancerDriver(
     def finalize_amphora(self, amphora):
         pass
 
-    def post_vip_plug(self, load_balancer, amphorae_network_config):
-        for amp in load_balancer.amphorae:
-            if amp.status != constants.DELETED:
-                subnet = amphorae_network_config.get(amp.id).vip_subnet
-                # NOTE(blogan): using the vrrp port here because that
-                # is what the allowed address pairs network driver sets
-                # this particular port to.  This does expose a bit of
-                # tight coupling between the network driver and amphora
-                # driver.  We will need to revisit this to try and remove
-                # this tight coupling.
-                port = amphorae_network_config.get(amp.id).vrrp_port
-                net_info = {'subnet_cidr': subnet.cidr,
-                            'gateway': subnet.gateway_ip,
-                            'mac_address': port.mac_address}
-                self.client.plug_vip(amp,
+    def post_vip_plug(self, amphora, load_balancer, amphorae_network_config):
+        if amphora.status != consts.DELETED:
+            subnet = amphorae_network_config.get(amphora.id).vip_subnet
+            # NOTE(blogan): using the vrrp port here because that
+            # is what the allowed address pairs network driver sets
+            # this particular port to.  This does expose a bit of
+            # tight coupling between the network driver and amphora
+            # driver.  We will need to revisit this to try and remove
+            # this tight coupling.
+            # NOTE (johnsom): I am loading the vrrp_ip into the
+            # net_info structure here so that I don't break
+            # compatibility with old amphora agent versions.
+
+            port = amphorae_network_config.get(amphora.id).vrrp_port
+            host_routes = [{'nexthop': hr.nexthop,
+                            'destination': hr.destination}
+                           for hr in subnet.host_routes]
+            net_info = {'subnet_cidr': subnet.cidr,
+                        'gateway': subnet.gateway_ip,
+                        'mac_address': port.mac_address,
+                        'vrrp_ip': amphora.vrrp_ip,
+                        'mtu': port.network.mtu,
+                        'host_routes': host_routes}
+            try:
+                self.client.plug_vip(amphora,
                                      load_balancer.vip.ip_address,
                                      net_info)
+            except exc.Conflict:
+                LOG.warning(_LW('VIP with MAC {mac} already exists on '
+                                'amphora, skipping post_vip_plug').format(
+                    mac=port.mac_address))
 
     def post_network_plug(self, amphora, port):
-        port_info = {'mac_address': port.mac_address}
-        self.client.plug_network(amphora, port_info)
+        fixed_ips = []
+        for fixed_ip in port.fixed_ips:
+            host_routes = [{'nexthop': hr.nexthop,
+                            'destination': hr.destination}
+                           for hr in fixed_ip.subnet.host_routes]
+            ip = {'ip_address': fixed_ip.ip_address,
+                  'subnet_cidr': fixed_ip.subnet.cidr,
+                  'host_routes': host_routes}
+            fixed_ips.append(ip)
+        port_info = {'mac_address': port.mac_address,
+                     'fixed_ips': fixed_ips,
+                     'mtu': port.network.mtu}
+        try:
+            self.client.plug_network(amphora, port_info)
+        except exc.Conflict:
+            LOG.warning(_LW('Network with MAC {mac} already exists on '
+                            'amphora, skipping post_network_plug').format(
+                mac=port.mac_address))
 
     def get_vrrp_interface(self, amphora):
         return self.client.get_interface(amphora, amphora.vrrp_ip)['interface']
@@ -159,7 +184,7 @@ class HaproxyAmphoraLoadBalancerDriver(
 
         for cert in certs:
             pem = cert_parser.build_pem(cert)
-            md5 = hashlib.md5(six.b(pem)).hexdigest()
+            md5 = hashlib.md5(six.b(pem)).hexdigest()  # nosec
             name = '{cn}.pem'.format(cn=cert.primary_cn)
             self._apply(self._upload_cert, listener, pem, md5, name)
 
@@ -195,13 +220,19 @@ class AmphoraAPIClient(object):
         self.delete = functools.partial(self.request, 'delete')
         self.head = functools.partial(self.request, 'head')
 
-        self.start_listener = functools.partial(self._action, 'start')
-        self.stop_listener = functools.partial(self._action, 'stop')
-        self.reload_listener = functools.partial(self._action, 'reload')
+        self.start_listener = functools.partial(self._action,
+                                                consts.AMP_ACTION_START)
+        self.stop_listener = functools.partial(self._action,
+                                               consts.AMP_ACTION_STOP)
+        self.reload_listener = functools.partial(self._action,
+                                                 consts.AMP_ACTION_RELOAD)
 
-        self.start_vrrp = functools.partial(self._vrrp_action, 'start')
-        self.stop_vrrp = functools.partial(self._vrrp_action, 'stop')
-        self.reload_vrrp = functools.partial(self._vrrp_action, 'reload')
+        self.start_vrrp = functools.partial(self._vrrp_action,
+                                            consts.AMP_ACTION_START)
+        self.stop_vrrp = functools.partial(self._vrrp_action,
+                                           consts.AMP_ACTION_STOP)
+        self.reload_vrrp = functools.partial(self._vrrp_action,
+                                             consts.AMP_ACTION_RELOAD)
 
         self.session = requests.Session()
         self.session.cert = CONF.haproxy_amphora.client_cert
@@ -209,6 +240,12 @@ class AmphoraAPIClient(object):
         self.session.mount('https://', self.ssl_adapter)
 
     def _base_url(self, ip):
+        if utils.is_ipv6_lla(ip):
+            ip = '[{ip}%{interface}]'.format(
+                ip=ip,
+                interface=CONF.haproxy_amphora.lb_network_interface)
+        elif utils.is_ipv6(ip):
+            ip = '[{ip}]'.format(ip=ip)
         return "https://{ip}:{port}/{version}/".format(
             ip=ip,
             port=CONF.haproxy_amphora.bind_port,
@@ -230,6 +267,8 @@ class AmphoraAPIClient(object):
 
         headers['User-Agent'] = OCTAVIA_API_CLIENT
         self.ssl_adapter.uuid = amp.id
+        retry_attempt = False
+        exception = None
         # Keep retrying
         for a in six.moves.xrange(CONF.haproxy_amphora.connection_max_retries):
             try:
@@ -239,14 +278,25 @@ class AmphoraAPIClient(object):
                         message="A true SSLContext object is not available"
                     )
                     r = _request(**reqargs)
-            except (requests.ConnectionError, requests.Timeout):
+                LOG.debug("Connected to amphora. Response: {resp}".format(
+                    resp=r))
+                # Give a 404 response one retry.  Flask/werkzeug is
+                # returning 404 on startup.
+                if r.status_code == 404 and retry_attempt is False:
+                    retry_attempt = True
+                    raise requests.ConnectionError
+                return r
+            except (requests.ConnectionError, requests.Timeout) as e:
+                exception = e
                 LOG.warning(_LW("Could not connect to instance. Retrying."))
                 time.sleep(CONF.haproxy_amphora.connection_retry_interval)
-                if a == CONF.haproxy_amphora.connection_max_retries - 1:
-                    raise driver_except.TimeOutException()
-            else:
-                return r
-        raise driver_except.UnavailableException()
+
+        LOG.error(_LE("Connection retries (currently set to %(max_retries)s) "
+                      "exhausted.  The amphora is unavailable. Reason: "
+                      "%(exception)s"),
+                  {'max_retries': CONF.haproxy_amphora.connection_max_retries,
+                   'exception': exception})
+        raise driver_except.TimeOutException()
 
     def upload_config(self, amp, listener_id, config):
         r = self.put(
