@@ -22,12 +22,14 @@ from wsmeext import pecan as wsme_pecan
 
 from octavia.api.v1.controllers import base
 from octavia.api.v1.controllers import listener
+from octavia.api.v1.controllers import load_balancer_statistics as lb_stats
 from octavia.api.v1.controllers import pool
 from octavia.api.v1.types import load_balancer as lb_types
 from octavia.common import constants
 from octavia.common import data_models
 from octavia.common import exceptions
 import octavia.common.validate as validate
+from octavia.db import api as db_api
 from octavia.db import prepare as db_prepare
 from octavia.i18n import _LI
 
@@ -73,14 +75,17 @@ class LoadBalancersController(base.BaseController):
             raise exceptions.ImmutableObject(resource=db_lb._name(),
                                              id=id)
 
-    def _create_load_balancer_graph(self, context, load_balancer):
+    def _create_load_balancer_graph_db(self, lock_session, load_balancer):
         prepped_lb = db_prepare.create_load_balancer_tree(
             load_balancer.to_dict(render_unsets=True))
         try:
             db_lb = self.repositories.create_load_balancer_tree(
-                context.session, prepped_lb)
+                lock_session, prepped_lb)
         except Exception:
             raise
+        return db_lb
+
+    def _load_balancer_graph_to_handler(self, context, db_lb):
         try:
             LOG.info(_LI("Sending full load balancer configuration %s to "
                          "the handler"), db_lb.id)
@@ -97,24 +102,49 @@ class LoadBalancersController(base.BaseController):
                          body=lb_types.LoadBalancerPOST, status_code=202)
     def post(self, load_balancer):
         """Creates a load balancer."""
+        context = pecan.request.context.get('octavia_context')
         # Validate the subnet id
         if load_balancer.vip.subnet_id:
             if not validate.subnet_exists(load_balancer.vip.subnet_id):
                 raise exceptions.NotFound(resource='Subnet',
                                           id=load_balancer.vip.subnet_id)
 
-        context = pecan.request.context.get('octavia_context')
+        lock_session = db_api.get_session(autocommit=False)
+        if self.repositories.check_quota_met(
+                context.session,
+                lock_session,
+                data_models.LoadBalancer,
+                load_balancer.project_id):
+            lock_session.rollback()
+            raise exceptions.QuotaException
+
         if load_balancer.listeners:
-            return self._create_load_balancer_graph(context, load_balancer)
-        lb_dict = db_prepare.create_load_balancer(load_balancer.to_dict(
-            render_unsets=True
-        ))
-        vip_dict = lb_dict.pop('vip', {})
+            try:
+                db_lb = self._create_load_balancer_graph_db(lock_session,
+                                                            load_balancer)
+                lock_session.commit()
+            except Exception:
+                with excutils.save_and_reraise_exception():
+                    lock_session.rollback()
+
+            return self._load_balancer_graph_to_handler(context, db_lb)
+
         try:
+            lb_dict = db_prepare.create_load_balancer(load_balancer.to_dict(
+                render_unsets=True
+            ))
+            vip_dict = lb_dict.pop('vip', {})
+
             db_lb = self.repositories.create_load_balancer_and_vip(
-                context.session, lb_dict, vip_dict)
+                lock_session, lb_dict, vip_dict)
+            lock_session.commit()
         except odb_exceptions.DBDuplicateEntry:
+            lock_session.rollback()
             raise exceptions.IDAlreadyExists()
+        except Exception:
+            with excutils.save_and_reraise_exception():
+                lock_session.rollback()
+
         # Handler will be responsible for sending to controller
         try:
             LOG.info(_LI("Sending created Load Balancer %s to the handler"),
@@ -178,9 +208,9 @@ class LoadBalancersController(base.BaseController):
         decides which controller, if any, should control be passed.
         """
         context = pecan.request.context.get('octavia_context')
-        if lb_id and len(remainder) and (remainder[0] == 'listeners' or
-                                         remainder[0] == 'pools' or
-                                         remainder[0] == 'delete_cascade'):
+
+        possible_remainder = ('listeners', 'pools', 'delete_cascade', 'stats')
+        if lb_id and len(remainder) and (remainder[0] in possible_remainder):
             controller = remainder[0]
             remainder = remainder[1:]
             db_lb = self.repositories.load_balancer.get(context.session,
@@ -197,6 +227,9 @@ class LoadBalancersController(base.BaseController):
                     load_balancer_id=db_lb.id), remainder
             elif (controller == 'delete_cascade'):
                 return LBCascadeDeleteController(db_lb.id), ''
+            elif (controller == 'stats'):
+                return lb_stats.LoadBalancerStatisticsController(
+                    loadbalancer_id=db_lb.id), remainder
 
 
 class LBCascadeDeleteController(LoadBalancersController):
