@@ -17,6 +17,7 @@
 Routines for configuring Octavia
 """
 
+from keystoneauth1 import loading as ks_loading
 from oslo_config import cfg
 from oslo_db import options as db_options
 from oslo_log import log as logging
@@ -34,14 +35,15 @@ core_opts = [
               help=_("The host IP to bind to")),
     cfg.PortOpt('bind_port', default=9876,
                 help=_("The port to bind to")),
+    cfg.StrOpt('auth_strategy', default=constants.NOAUTH,
+               choices=[constants.NOAUTH, constants.KEYSTONE],
+               help=_("The auth strategy for API requests.")),
     cfg.StrOpt('api_handler', default='queue_producer',
                help=_("The handler that the API communicates with")),
     cfg.StrOpt('api_paste_config', default="api-paste.ini",
                help=_("The API paste config file to use")),
     cfg.StrOpt('api_extensions_path', default="",
                help=_("The path for API extensions")),
-    cfg.StrOpt('auth_strategy', default='keystone',
-               help=_("The type of authentication to use")),
     cfg.BoolOpt('allow_bulk', default=True,
                 help=_("Allow the usage of the bulk API")),
     cfg.BoolOpt('allow_pagination', default=False,
@@ -67,14 +69,15 @@ amphora_agent_opts = [
                help=_("The server certificate for the agent.py server "
                       "to use")),
     cfg.StrOpt('agent_server_network_dir',
-               default='/etc/netns/{}/network/interfaces.d/'.format(
-                   constants.AMPHORA_NAMESPACE),
                help=_("The directory where new network interfaces "
                       "are located")),
     cfg.StrOpt('agent_server_network_file',
                help=_("The file where the network interfaces are located. "
                       "Specifying this will override any value set for "
                       "agent_server_network_dir.")),
+    cfg.IntOpt('agent_request_read_timeout', default=120,
+               help=_("The time in seconds to allow a request from the "
+                      "controller to run before terminating the socket.")),
     # Do not specify in octavia.conf, loaded at runtime
     cfg.StrOpt('amphora_id', help=_("The amphora ID.")),
 ]
@@ -142,13 +145,6 @@ oslo_messaging_opts = [
                help=_('topic name for communicating events through a queue')),
 ]
 
-keystone_authtoken_v3_opts = [
-    cfg.StrOpt('admin_user_domain', default='default',
-               help=_('Admin user keystone authentication domain')),
-    cfg.StrOpt('admin_project_domain', default='default',
-               help=_('Admin project keystone authentication domain'))
-]
-
 haproxy_amphora_opts = [
     cfg.StrOpt('base_path',
                default='/var/lib/octavia',
@@ -164,15 +160,23 @@ haproxy_amphora_opts = [
                default=5,
                help=_('Retry timeout between connection attempts in '
                       'seconds.')),
+    cfg.StrOpt('user_group',
+               default='nogroup',
+               help=_('The user group for haproxy to run under inside the '
+                      'amphora.')),
     cfg.StrOpt('haproxy_stick_size', default='10k',
                help=_('Size of the HAProxy stick table. Accepts k, m, g '
                       'suffixes.  Example: 10k')),
 
     # REST server
-    cfg.IPOpt('bind_host', default='0.0.0.0',  # nosec
+    cfg.IPOpt('bind_host', default='::',  # nosec
               help=_("The host IP to bind to")),
     cfg.PortOpt('bind_port', default=9443,
                 help=_("The port to bind to")),
+    cfg.StrOpt('lb_network_interface',
+               default='o-hm0',
+               help=_('Network interface through which to reach amphora, only '
+                      'required if using IPv6 link local addresses.')),
     cfg.StrOpt('haproxy_cmd', default='/usr/sbin/haproxy',
                help=_("The full path to haproxy")),
     cfg.IntOpt('respawn_count', default=2,
@@ -191,6 +195,9 @@ haproxy_amphora_opts = [
     cfg.StrOpt('server_ca', default='/etc/octavia/certs/server_ca.pem',
                help=_("The ca which signed the server certificates")),
     cfg.BoolOpt('use_upstart', default=True,
+                deprecated_for_removal=True,
+                deprecated_reason='This is now automatically discovered '
+                                  ' and configured.',
                 help=_("If False, use sysvinit.")),
 ]
 
@@ -252,9 +259,6 @@ controller_worker_opts = [
     cfg.StrOpt('network_driver',
                default='network_noop_driver',
                help=_('Name of the network driver to use')),
-    cfg.StrOpt('cert_generator',
-               default='local_cert_generator',
-               help=_('Name of the cert generator to use')),
     cfg.StrOpt('loadbalancer_topology',
                default=constants.TOPOLOGY_SINGLE,
                choices=constants.SUPPORTED_LB_TOPOLOGIES,
@@ -289,12 +293,22 @@ certificate_opts = [
     cfg.StrOpt('barbican_auth',
                default='barbican_acl_auth',
                help='Name of the Barbican authentication method to use'),
+    cfg.StrOpt('service_name',
+               help=_('The name of the certificate service in the keystone'
+                      'catalog')),
+    cfg.StrOpt('endpoint', help=_('A new endpoint to override the endpoint '
+                                  'in the keystone catalog.')),
     cfg.StrOpt('region_name',
                help='Region in Identity service catalog to use for '
                     'communication with the barbican service.'),
     cfg.StrOpt('endpoint_type',
                default='publicURL',
-               help='The endpoint_type to be used for barbican service.')
+               help='The endpoint_type to be used for barbican service.'),
+    cfg.StrOpt('ca_certificates_file',
+               help=_('CA certificates file path')),
+    cfg.BoolOpt('insecure',
+                default=False,
+                help=_('Disable certificate validation on SSL connections ')),
 ]
 
 house_keeping_opts = [
@@ -420,6 +434,24 @@ glance_opts = [
                 help=_('Disable certificate validation on SSL connections ')),
 ]
 
+quota_opts = [
+    cfg.IntOpt('default_load_balancer_quota',
+               default=constants.QUOTA_UNLIMITED,
+               help=_('Default per project load balancer quota.')),
+    cfg.IntOpt('default_listener_quota',
+               default=constants.QUOTA_UNLIMITED,
+               help=_('Default per project listener quota.')),
+    cfg.IntOpt('default_member_quota',
+               default=constants.QUOTA_UNLIMITED,
+               help=_('Default per project member quota.')),
+    cfg.IntOpt('default_pool_quota',
+               default=constants.QUOTA_UNLIMITED,
+               help=_('Default per project pool quota.')),
+    cfg.IntOpt('default_health_monitor_quota',
+               default=constants.QUOTA_UNLIMITED,
+               help=_('Default per project health monitor quota.')),
+]
+
 
 # Register the configuration options
 cfg.CONF.register_opts(core_opts)
@@ -436,12 +468,10 @@ cfg.CONF.register_opts(anchor_opts, group='anchor')
 cfg.CONF.register_cli_opts(core_cli_opts)
 cfg.CONF.register_opts(certificate_opts, group='certificates')
 cfg.CONF.register_cli_opts(healthmanager_opts, group='health_manager')
-cfg.CONF.import_group('keystone_authtoken', 'keystonemiddleware.auth_token')
-cfg.CONF.register_opts(keystone_authtoken_v3_opts,
-                       group='keystone_authtoken_v3')
 cfg.CONF.register_opts(nova_opts, group='nova')
 cfg.CONF.register_opts(glance_opts, group='glance')
 cfg.CONF.register_opts(neutron_opts, group='neutron')
+cfg.CONF.register_opts(quota_opts, group='quotas')
 
 
 # Ensure that the control exchange is set correctly
@@ -454,6 +484,9 @@ db_options.set_defaults(cfg.CONF, connection=_SQL_CONNECTION_DEFAULT,
 
 logging.register_options(cfg.CONF)
 
+ks_loading.register_auth_conf_options(cfg.CONF, constants.SERVICE_AUTH)
+ks_loading.register_session_conf_options(cfg.CONF, constants.SERVICE_AUTH)
+
 
 def init(args, **kwargs):
     cfg.CONF(args=args, project='octavia',
@@ -469,30 +502,3 @@ def setup_logging(conf):
     product_name = "octavia"
     logging.setup(conf, product_name)
     LOG.info(_LI("Logging enabled!"))
-
-
-# def load_paste_app(app_name):
-#     """Builds and returns a WSGI app from a paste config file.
-
-#     :param app_name: Name of the application to load
-#     :raises ConfigFilesNotFoundError when config file cannot be located
-#     :raises RuntimeError when application cannot be loaded from config file
-#     """
-
-#     config_path = cfg.CONF.find_file(cfg.CONF.api_paste_config)
-#     if not config_path:
-#         raise cfg.ConfigFilesNotFoundError(
-#             config_files=[cfg.CONF.api_paste_config])
-#     config_path = os.path.abspath(config_path)
-#     LOG.info(_LI("Config paste file: %s"), config_path)
-
-#     try:
-#         app = deploy.loadapp("config:%s" % config_path, name=app_name)
-#     except (LookupError, ImportError):
-#         msg = (_("Unable to load %(app_name)s from "
-#                  "configuration file %(config_path)s.") %
-#                {'app_name': app_name,
-#                 'config_path': config_path})
-#         LOG.exception(msg)
-#         raise RuntimeError(msg)
-#     return app
