@@ -32,6 +32,9 @@ HEALTH_MONITOR_HTTP = 'HTTP'
 HEALTH_MONITOR_HTTPS = 'HTTPS'
 SUPPORTED_HEALTH_MONITOR_TYPES = (HEALTH_MONITOR_HTTP, HEALTH_MONITOR_HTTPS,
                                   HEALTH_MONITOR_PING, HEALTH_MONITOR_TCP)
+HEALTH_MONITOR_HTTP_DEFAULT_METHOD = 'GET'
+HEALTH_MONITOR_DEFAULT_EXPECTED_CODES = '200'
+HEALTH_MONITOR_DEFAULT_URL_PATH = '/'
 
 UPDATE_STATS = 'UPDATE_STATS'
 UPDATE_HEALTH = 'UPDATE_HEALTH'
@@ -63,6 +66,7 @@ SUPPORTED_PROVISIONING_STATUSES = (ACTIVE, AMPHORA_ALLOCATED,
                                    PENDING_DELETE, PENDING_CREATE,
                                    PENDING_UPDATE, DELETED, ERROR)
 MUTABLE_STATUSES = (ACTIVE,)
+DELETABLE_STATUSES = (ACTIVE, ERROR)
 
 SUPPORTED_AMPHORA_STATUSES = (AMPHORA_ALLOCATED, AMPHORA_BOOTING,
                               AMPHORA_READY, DELETED, PENDING_DELETE)
@@ -157,6 +161,7 @@ AMPS_DATA = 'amps_data'
 NICS = 'nics'
 VIP = 'vip'
 POOL = 'pool'
+POOL_CHILD_COUNT = 'pool_child_count'
 POOL_ID = 'pool_id'
 L7POLICY = 'l7policy'
 L7RULE = 'l7rule'
@@ -202,9 +207,6 @@ UPDATE_MEMBER_FLOW = 'octavia-update-member-flow'
 UPDATE_POOL_FLOW = 'octavia-update-pool-flow'
 UPDATE_L7POLICY_FLOW = 'octavia-update-l7policy-flow'
 UPDATE_L7RULE_FLOW = 'octavia-update-l7rule-flow'
-WAIT_FOR_AMPHORA = 'wait-for-amphora'
-
-FAILOVER_AMPHORA_FLOW = 'octavia-failover-amphora-flow'
 
 POST_MAP_AMP_TO_LB_SUBFLOW = 'octavia-post-map-amp-to-lb-subflow'
 CREATE_AMP_FOR_LB_SUBFLOW = 'octavia-create-amp-for-lb-subflow'
@@ -247,9 +249,12 @@ RELOAD_LB_AFTER_AMP_ASSOC_FULL_GRAPH = 'reload-lb-after-amp-assoc-full-graph'
 RELOAD_LB_AFTER_PLUG_VIP = 'reload-lb-after-plug-vip'
 
 NOVA_1 = '1.1'
-NOVA_2 = '2'
+NOVA_21 = '2.1'
 NOVA_3 = '3'
-NOVA_VERSIONS = (NOVA_1, NOVA_2, NOVA_3)
+NOVA_VERSIONS = (NOVA_1, NOVA_21, NOVA_3)
+
+# Auth sections
+SERVICE_AUTH = 'service_auth'
 
 RPC_NAMESPACE_CONTROLLER_AGENT = 'controller'
 
@@ -306,10 +311,63 @@ NO_CHECK = 'no check'
 
 HAPROXY_MEMBER_STATUSES = (UP, DOWN, NO_CHECK)
 
+# Quota Constants
+QUOTA_UNLIMITED = -1
+
 API_VERSION = '0.5'
 
 HAPROXY_BASE_PEER_PORT = 1025
-KEEPALIVED_CONF = 'keepalived.conf.j2'
+KEEPALIVED_JINJA2_UPSTART = 'keepalived.upstart.j2'
+KEEPALIVED_JINJA2_SYSTEMD = 'keepalived.systemd.j2'
+KEEPALIVED_JINJA2_SYSVINIT = 'keepalived.sysvinit.j2'
 CHECK_SCRIPT_CONF = 'keepalived_check_script.conf.j2'
+
 PLUGGED_INTERFACES = '/var/lib/octavia/plugged_interfaces'
 AMPHORA_NAMESPACE = 'amphora-haproxy'
+
+# List of HTTP headers which are supported for insertion
+SUPPORTED_HTTP_HEADERS = ['X-Forwarded-For',
+                          'X-Forwarded-Port']
+
+FLOW_DOC_TITLES = {'AmphoraFlows': 'Amphora Flows',
+                   'LoadBalancerFlows': 'Load Balancer Flows',
+                   'ListenerFlows': 'Listener Flows',
+                   'PoolFlows': 'Pool Flows',
+                   'MemberFlows': 'Member Flows',
+                   'HealthMonitorFlows': 'Health Monitor Flows',
+                   'L7PolicyFlows': 'Layer 7 Policy Flows',
+                   'L7RuleFlows': 'Layer 7 Rule Flows'}
+
+NETNS_PRIMARY_INTERFACE = 'eth1'
+SYSCTL_CMD = '/sbin/sysctl'
+
+AMP_ACTION_START = 'start'
+AMP_ACTION_STOP = 'stop'
+AMP_ACTION_RELOAD = 'reload'
+GLANCE_IMAGE_ACTIVE = 'active'
+
+INIT_SYSTEMD = 'systemd'
+INIT_UPSTART = 'upstart'
+INIT_SYSVINIT = 'sysvinit'
+INIT_UNKOWN = 'unknown'
+VALID_INIT_SYSTEMS = (INIT_SYSTEMD, INIT_SYSVINIT, INIT_UPSTART)
+INIT_PATH = '/sbin/init'
+
+SYSTEMD_DIR = '/usr/lib/systemd/system'
+SYSVINIT_DIR = '/etc/init.d'
+UPSTART_DIR = '/etc/init'
+
+INIT_PROC_COMM_PATH = '/proc/1/comm'
+
+KEEPALIVED_SYSTEMD = 'octavia-keepalived.service'
+KEEPALIVED_SYSVINIT = 'octavia-keepalived'
+KEEPALIVED_UPSTART = 'octavia-keepalived.conf'
+
+# Authentication
+KEYSTONE = 'keystone'
+NOAUTH = 'noauth'
+
+UBUNTU_AMP_NET_DIR_TEMPLATE = '/etc/netns/{netns}/network/interfaces.d/'
+RH_AMP_NET_DIR_TEMPLATE = '/etc/netns/{netns}/sysconfig/network-scripts/'
+UBUNTU = 'ubuntu'
+CENTOS = 'centos'
