@@ -135,15 +135,58 @@ class SessionPersistence(BaseDataModel):
 
 class ListenerStatistics(BaseDataModel):
 
-    def __init__(self, listener_id=None, amphora_id=None, bytes_in=None,
-                 bytes_out=None, active_connections=None,
-                 total_connections=None):
+    def __init__(self, listener_id=None, amphora_id=None, bytes_in=0,
+                 bytes_out=0, active_connections=0,
+                 total_connections=0, request_errors=0):
         self.listener_id = listener_id
         self.amphora_id = amphora_id
         self.bytes_in = bytes_in
         self.bytes_out = bytes_out
         self.active_connections = active_connections
         self.total_connections = total_connections
+        self.request_errors = request_errors
+
+    def get_stats(self):
+        stats = {
+            'bytes_in': self.bytes_in,
+            'bytes_out': self.bytes_out,
+            'active_connections': self.active_connections,
+            'total_connections': self.total_connections,
+            'request_errors': self.request_errors,
+        }
+        return stats
+
+    def __iadd__(self, other):
+
+        if isinstance(other, ListenerStatistics):
+            self.bytes_in += other.bytes_in
+            self.bytes_out += other.bytes_out
+            self.request_errors += other.request_errors
+            self.total_connections += other.total_connections
+
+        return self
+
+
+class LoadBalancerStatistics(BaseDataModel):
+
+    def __init__(self, bytes_in=0, bytes_out=0, active_connections=0,
+                 total_connections=0, request_errors=0, listeners=None):
+        self.bytes_in = bytes_in
+        self.bytes_out = bytes_out
+        self.active_connections = active_connections
+        self.total_connections = total_connections
+        self.request_errors = request_errors
+        self.listeners = listeners or []
+
+    def get_stats(self):
+        stats = {
+            'bytes_in': self.bytes_in,
+            'bytes_out': self.bytes_out,
+            'active_connections': self.active_connections,
+            'total_connections': self.total_connections,
+            'request_errors': self.request_errors,
+        }
+        return stats
 
 
 class HealthMonitor(BaseDataModel):
@@ -152,7 +195,7 @@ class HealthMonitor(BaseDataModel):
                  delay=None, timeout=None, fall_threshold=None,
                  rise_threshold=None, http_method=None, url_path=None,
                  expected_codes=None, enabled=None, pool=None,
-                 provisioning_status=None):
+                 provisioning_status=None, name=None):
         self.id = id
         self.project_id = project_id
         self.pool_id = pool_id
@@ -167,6 +210,7 @@ class HealthMonitor(BaseDataModel):
         self.enabled = enabled
         self.pool = pool
         self.provisioning_status = provisioning_status
+        self.name = name
 
     def delete(self):
         self.pool.health_monitor = None
@@ -241,7 +285,8 @@ class Member(BaseDataModel):
     def __init__(self, id=None, project_id=None, pool_id=None, ip_address=None,
                  protocol_port=None, weight=None, enabled=None,
                  subnet_id=None, operating_status=None, pool=None,
-                 created_at=None, updated_at=None, provisioning_status=None):
+                 created_at=None, updated_at=None, provisioning_status=None,
+                 name=None):
         self.id = id
         self.project_id = project_id
         self.pool_id = pool_id
@@ -255,6 +300,7 @@ class Member(BaseDataModel):
         self.created_at = created_at
         self.updated_at = updated_at
         self.provisioning_status = provisioning_status
+        self.name = name
 
     def delete(self):
         for mem in self.pool.members:
@@ -561,3 +607,30 @@ class L7Policy(BaseDataModel):
             if p.id == self.id:
                 self.listener.l7policies.remove(p)
                 break
+
+
+class Quotas(BaseDataModel):
+
+    def __init__(self,
+                 project_id=None,
+                 load_balancer=None,
+                 listener=None,
+                 pool=None,
+                 health_monitor=None,
+                 member=None,
+                 in_use_health_monitor=None,
+                 in_use_listener=None,
+                 in_use_load_balancer=None,
+                 in_use_member=None,
+                 in_use_pool=None):
+        self.project_id = project_id
+        self.health_monitor = health_monitor
+        self.listener = listener
+        self.load_balancer = load_balancer
+        self.pool = pool
+        self.member = member
+        self.in_use_health_monitor = in_use_health_monitor
+        self.in_use_listener = in_use_listener
+        self.in_use_load_balancer = in_use_load_balancer
+        self.in_use_member = in_use_member
+        self.in_use_pool = in_use_pool
