@@ -26,8 +26,6 @@ from octavia.controller.worker.tasks import lifecycle_tasks
 from octavia.controller.worker.tasks import network_tasks
 
 CONF = cfg.CONF
-CONF.import_group('controller_worker', 'octavia.common.config')
-CONF.import_group('nova', 'octavia.common.config')
 
 
 class AmphoraFlows(object):
@@ -276,7 +274,8 @@ class AmphoraFlows(object):
                                     requires=constants.AMPHORA))
         return delete_amphora_flow
 
-    def get_failover_flow(self, role=constants.ROLE_STANDALONE):
+    def get_failover_flow(self, role=constants.ROLE_STANDALONE,
+                          status=constants.AMPHORA_READY):
         """Creates a flow to failover a stale amphora
 
         :returns: The flow for amphora failover
@@ -312,6 +311,10 @@ class AmphoraFlows(object):
             rebind={constants.AMPHORA: constants.FAILED_AMPHORA},
             requires=constants.AMPHORA))
 
+        # If this is an unallocated amp (spares pool), we're done
+        if status != constants.AMPHORA_ALLOCATED:
+            return failover_amphora_flow
+
         # Save failed amphora details for later
         failover_amphora_flow.add(
             database_tasks.GetAmphoraDetails(
@@ -351,8 +354,7 @@ class AmphoraFlows(object):
 
         # Plug the member networks into the new amphora
         failover_amphora_flow.add(network_tasks.CalculateDelta(
-            requires=(constants.LOADBALANCER),
-            provides=constants.DELTAS))
+            requires=constants.LOADBALANCER, provides=constants.DELTAS))
         failover_amphora_flow.add(network_tasks.HandleNetworkDeltas(
             requires=constants.DELTAS, provides=constants.ADDED_PORTS))
         failover_amphora_flow.add(amphora_driver_tasks.AmphoraePostNetworkPlug(
