@@ -18,6 +18,7 @@ from taskflow.patterns import linear_flow
 from octavia.common import constants
 from octavia.controller.worker.tasks import amphora_driver_tasks
 from octavia.controller.worker.tasks import database_tasks
+from octavia.controller.worker.tasks import lifecycle_tasks
 from octavia.controller.worker.tasks import model_tasks
 
 
@@ -29,8 +30,16 @@ class HealthMonitorFlows(object):
         :returns: The flow for creating a health monitor
         """
         create_hm_flow = linear_flow.Flow(constants.CREATE_HEALTH_MONITOR_FLOW)
+        create_hm_flow.add(lifecycle_tasks.HealthMonitorToErrorOnRevertTask(
+            requires=[constants.HEALTH_MON,
+                      constants.LISTENERS,
+                      constants.LOADBALANCER]))
+        create_hm_flow.add(database_tasks.MarkHealthMonitorPendingCreateInDB(
+            requires=constants.HEALTH_MON))
         create_hm_flow.add(amphora_driver_tasks.ListenersUpdate(
             requires=[constants.LOADBALANCER, constants.LISTENERS]))
+        create_hm_flow.add(database_tasks.MarkHealthMonitorActiveInDB(
+            requires=constants.HEALTH_MON))
         create_hm_flow.add(database_tasks.MarkLBAndListenersActiveInDB(
             requires=[constants.LOADBALANCER, constants.LISTENERS]))
 
@@ -42,6 +51,12 @@ class HealthMonitorFlows(object):
         :returns: The flow for deleting a health monitor
         """
         delete_hm_flow = linear_flow.Flow(constants.DELETE_HEALTH_MONITOR_FLOW)
+        delete_hm_flow.add(lifecycle_tasks.HealthMonitorToErrorOnRevertTask(
+            requires=[constants.HEALTH_MON,
+                      constants.LISTENERS,
+                      constants.LOADBALANCER]))
+        delete_hm_flow.add(database_tasks.MarkHealthMonitorPendingDeleteInDB(
+            requires=constants.HEALTH_MON))
         delete_hm_flow.add(model_tasks.
                            DeleteModelObject(rebind={constants.OBJECT:
                                                      constants.HEALTH_MON}))
@@ -49,6 +64,8 @@ class HealthMonitorFlows(object):
             requires=[constants.LOADBALANCER, constants.LISTENERS]))
         delete_hm_flow.add(database_tasks.DeleteHealthMonitorInDB(
             requires=constants.POOL_ID))
+        delete_hm_flow.add(database_tasks.DecrementHealthMonitorQuota(
+            requires=constants.HEALTH_MON))
         delete_hm_flow.add(database_tasks.MarkLBAndListenersActiveInDB(
             requires=[constants.LOADBALANCER, constants.LISTENERS]))
 
@@ -60,6 +77,12 @@ class HealthMonitorFlows(object):
         :returns: The flow for updating a health monitor
         """
         update_hm_flow = linear_flow.Flow(constants.UPDATE_HEALTH_MONITOR_FLOW)
+        update_hm_flow.add(lifecycle_tasks.HealthMonitorToErrorOnRevertTask(
+            requires=[constants.HEALTH_MON,
+                      constants.LISTENERS,
+                      constants.LOADBALANCER]))
+        update_hm_flow.add(database_tasks.MarkHealthMonitorPendingUpdateInDB(
+            requires=constants.HEALTH_MON))
         update_hm_flow.add(model_tasks.
                            UpdateAttributes(
                                rebind={constants.OBJECT: constants.HEALTH_MON},
@@ -68,6 +91,8 @@ class HealthMonitorFlows(object):
             requires=[constants.LOADBALANCER, constants.LISTENERS]))
         update_hm_flow.add(database_tasks.UpdateHealthMonInDB(
             requires=[constants.HEALTH_MON, constants.UPDATE_DICT]))
+        update_hm_flow.add(database_tasks.MarkHealthMonitorActiveInDB(
+            requires=constants.HEALTH_MON))
         update_hm_flow.add(database_tasks.MarkLBAndListenersActiveInDB(
             requires=[constants.LOADBALANCER, constants.LISTENERS]))
 
