@@ -21,18 +21,24 @@ reference
 import datetime
 
 from oslo_config import cfg
+from oslo_db import exception as db_exception
+from oslo_log import log as logging
+from oslo_utils import excutils
 from oslo_utils import uuidutils
+import six
 
-from octavia.common import constants
+from octavia.common import constants as consts
 from octavia.common import data_models
 from octavia.common import exceptions
 from octavia.common import validate
+from octavia.db import api as db_api
 from octavia.db import models
+from octavia.i18n import _LE, _LW
 
 
 CONF = cfg.CONF
-CONF.import_group('health_manager', 'octavia.common.config')
-CONF.import_group('house_keeping', 'octavia.common.config')
+
+LOG = logging.getLogger(__name__)
 
 
 class BaseRepository(object):
@@ -115,7 +121,7 @@ class BaseRepository(object):
         """Determines whether an entity exists in the database by its id.
 
         :param session: A Sql Alchemy database session.
-        :param id: id of entity to check for existance.
+        :param id: id of entity to check for existence.
         :returns: octavia.common.data_model
         """
         return bool(session.query(self.model_class).filter_by(id=id).first())
@@ -137,6 +143,7 @@ class Repositories(object):
         self.vrrpgroup = VRRPGroupRepository()
         self.l7rule = L7RuleRepository()
         self.l7policy = L7PolicyRepository()
+        self.quotas = QuotasRepository()
 
     def create_load_balancer_and_vip(self, session, lb_dict, vip_dict):
         """Inserts load balancer and vip entities into the database.
@@ -239,67 +246,368 @@ class Repositories(object):
                                  provisioning_status=listener_prov_status)
         return success
 
+    def check_quota_met(self, session, lock_session, _class, project_id):
+        """Checks and updates object quotas.
+
+        This method makes sure the project has available quota
+        for the resource and updates the quota to reflect the
+        new ussage.
+
+        :param session: Context database session
+        :param lock_session: Locking database session (autocommit=False)
+        :param _class: Data model object requesting quota
+        :param project_id: Project ID requesting quota
+        :returns: True if quota is met, False if quota was available
+        """
+        LOG.debug('Checking quota for project: {proj} object: {obj}'.format(
+            proj=project_id, obj=str(_class)))
+
+        # Under noauth everything is admin, so no quota
+        if CONF.auth_strategy == consts.NOAUTH:
+            LOG.debug('Auth strategy is NOAUTH, skipping quota check.')
+            return False
+
+        if not project_id:
+            raise exceptions.MissingProjectID()
+
+        quotas = self.quotas.get(session, project_id=project_id)
+        if not quotas:
+            # Make sure we have a record to lock
+            quotas = self.quotas.update(
+                session,
+                project_id,
+                quota={})
+            session.flush()
+        # Lock the project record in the database to block other quota checks
+        #
+        # Note: You cannot just use the current count as the in-use
+        # value as we don't want to lock the whole resource table
+        try:
+            quotas = lock_session.query(models.Quotas).filter_by(
+                project_id=project_id).with_for_update().first()
+            if _class == data_models.LoadBalancer:
+                # Decide which quota to use
+                if quotas.load_balancer is None:
+                    lb_quota = CONF.quotas.default_load_balancer_quota
+                else:
+                    lb_quota = quotas.load_balancer
+                # Get the current in use count
+                if not quotas.in_use_load_balancer:
+                    # This is to handle the upgrade case
+                    lb_count = session.query(models.LoadBalancer).filter(
+                        models.LoadBalancer.project_id == project_id,
+                        models.LoadBalancer.provisioning_status !=
+                        consts.DELETED).count() + 1
+                else:
+                    lb_count = quotas.in_use_load_balancer + 1
+                # Decide if the quota is met
+                if lb_count <= lb_quota or lb_quota == consts.QUOTA_UNLIMITED:
+                    quotas.in_use_load_balancer = lb_count
+                    return False
+                else:
+                    return True
+            if _class == data_models.Listener:
+                # Decide which quota to use
+                if quotas.listener is None:
+                    listener_quota = CONF.quotas.default_listener_quota
+                else:
+                    listener_quota = quotas.listener
+                # Get the current in use count
+                if not quotas.in_use_listener:
+                    # This is to handle the upgrade case
+                    listener_count = session.query(models.Listener).filter(
+                        models.Listener.project_id == project_id,
+                        models.Listener.provisioning_status !=
+                        consts.DELETED).count() + 1
+                else:
+                    listener_count = quotas.in_use_listener + 1
+                # Decide if the quota is met
+                if (listener_count <= listener_quota or
+                        listener_quota == consts.QUOTA_UNLIMITED):
+                    quotas.in_use_listener = listener_count
+                    return False
+                else:
+                    return True
+            if _class == data_models.Pool:
+                # Decide which quota to use
+                if quotas.pool is None:
+                    pool_quota = CONF.quotas.default_pool_quota
+                else:
+                    pool_quota = quotas.pool
+                # Get the current in use count
+                if not quotas.in_use_pool:
+                    # This is to handle the upgrade case
+                    pool_count = session.query(models.Pool).filter(
+                        models.Pool.project_id == project_id,
+                        models.Pool.provisioning_status !=
+                        consts.DELETED).count() + 1
+                else:
+                    pool_count = quotas.in_use_pool + 1
+                # Decide if the quota is met
+                if (pool_count <= pool_quota or
+                        pool_quota == consts.QUOTA_UNLIMITED):
+                    quotas.in_use_pool = pool_count
+                    return False
+                else:
+                    return True
+            if _class == data_models.HealthMonitor:
+                # Decide which quota to use
+                if quotas.health_monitor is None:
+                    hm_quota = CONF.quotas.default_health_monitor_quota
+                else:
+                    hm_quota = quotas.health_monitor
+                # Get the current in use count
+                if not quotas.in_use_health_monitor:
+                    # This is to handle the upgrade case
+                    hm_count = session.query(models.HealthMonitor).filter(
+                        models.HealthMonitor.project_id == project_id,
+                        models.HealthMonitor.provisioning_status !=
+                        consts.DELETED).count() + 1
+                else:
+                    hm_count = quotas.in_use_health_monitor + 1
+                # Decide if the quota is met
+                if (hm_count <= hm_quota or
+                        hm_quota == consts.QUOTA_UNLIMITED):
+                    quotas.in_use_health_monitor = hm_count
+                    return False
+                else:
+                    return True
+            if _class == data_models.Member:
+                # Decide which quota to use
+                if quotas.member is None:
+                    member_quota = CONF.quotas.default_member_quota
+                else:
+                    member_quota = quotas.member
+                # Get the current in use count
+                if not quotas.in_use_member:
+                    # This is to handle the upgrade case
+                    member_count = session.query(models.Member).filter(
+                        models.Member.project_id == project_id,
+                        models.Member.provisioning_status !=
+                        consts.DELETED).count() + 1
+                else:
+                    member_count = quotas.in_use_member + 1
+                # Decide if the quota is met
+                if (member_count <= member_quota or
+                        member_quota == consts.QUOTA_UNLIMITED):
+                    quotas.in_use_member = member_count
+                    return False
+                else:
+                    return True
+        except db_exception.DBDeadlock:
+            LOG.warning(_LW('Quota project lock timed out for project: '
+                            '{proj}').format(proj=project_id))
+            raise exceptions.ProjectBusyException()
+        return False
+
+    def decrement_quota(self, lock_session, _class, project_id, quantity=1):
+        """Decrements the object quota for a project
+
+        :param lock_session: Locking database session (autocommit=False)
+        :param _class: Data model object to decrement quota
+        :param project_id: Project ID to decrement quota on
+        :param quantity: Quantity of quota to decrement
+        :returns: None
+        """
+        LOG.debug('Decrementing quota by: {quant} for project: {proj} '
+                  'object: {obj}'.format(quant=quantity, proj=project_id,
+                                         obj=str(_class)))
+
+        if not project_id:
+            raise exceptions.MissingProjectID()
+
+        # Lock the project record in the database to block other quota checks
+        try:
+            quotas = lock_session.query(models.Quotas).filter_by(
+                project_id=project_id).with_for_update().first()
+            if not quotas:
+                if not CONF.auth_strategy == consts.NOAUTH:
+                    LOG.error(_LE(
+                        'Quota decrement on {clss} called on project: {proj} '
+                        'with no quota record in the database.').format(
+                            clss=type(_class), proj=project_id))
+                return
+            if _class == data_models.LoadBalancer:
+                if quotas.in_use_load_balancer > 0:
+                    quotas.in_use_load_balancer = (
+                        quotas.in_use_load_balancer - quantity)
+                else:
+                    if not CONF.auth_strategy == consts.NOAUTH:
+                        LOG.warning(_LW(
+                            'Quota decrement on {clss} called on project: '
+                            '{proj} that would cause a negative '
+                            'quota.').format(clss=type(_class),
+                                             proj=project_id))
+            if _class == data_models.Listener:
+                if quotas.in_use_listener > 0:
+                    quotas.in_use_listener = (
+                        quotas.in_use_listener - quantity)
+                else:
+                    if not CONF.auth_strategy == consts.NOAUTH:
+                        LOG.warning(_LW(
+                            'Quota decrement on {clss} called on project: '
+                            '{proj} that would cause a negative '
+                            'quota.').format(clss=type(_class),
+                                             proj=project_id))
+            if _class == data_models.Pool:
+                if quotas.in_use_pool > 0:
+                    quotas.in_use_pool = (
+                        quotas.in_use_pool - quantity)
+                else:
+                    if not CONF.auth_strategy == consts.NOAUTH:
+                        LOG.warning(_LW(
+                            'Quota decrement on {clss} called on project: '
+                            '{proj} that would cause a negative '
+                            'quota.').format(clss=type(_class),
+                                             proj=project_id))
+            if _class == data_models.HealthMonitor:
+                if quotas.in_use_health_monitor > 0:
+                    quotas.in_use_health_monitor = (
+                        quotas.in_use_health_monitor - quantity)
+                else:
+                    if not CONF.auth_strategy == consts.NOAUTH:
+                        LOG.warning(_LW(
+                            'Quota decrement on {clss} called on project: '
+                            '{proj} that would cause a negative '
+                            'quota.').format(clss=type(_class),
+                                             proj=project_id))
+            if _class == data_models.Member:
+                if quotas.in_use_member > 0:
+                    quotas.in_use_member = (
+                        quotas.in_use_member - quantity)
+                else:
+                    if not CONF.auth_strategy == consts.NOAUTH:
+                        LOG.warning(_LW(
+                            'Quota decrement on {clss} called on project: '
+                            '{proj} that would cause a negative '
+                            'quota.').format(clss=type(_class),
+                                             proj=project_id))
+        except db_exception.DBDeadlock:
+            LOG.warning(_LW('Quota project lock timed out for project: '
+                            '{proj}').format(proj=project_id))
+            raise exceptions.ProjectBusyException()
+
     def create_load_balancer_tree(self, session, lb_dict):
         listener_dicts = lb_dict.pop('listeners', [])
         vip_dict = lb_dict.pop('vip')
         with session.begin(subtransactions=True):
-            lb_dm = self.create_load_balancer_and_vip(
-                session, lb_dict, vip_dict)
-            for listener_dict in listener_dicts:
-                pool_dict = listener_dict.pop('default_pool', None)
-                l7policies_dict = listener_dict.pop('l7policies', None)
-                sni_containers = listener_dict.pop('sni_containers', [])
-                if pool_dict:
-                    hm_dict = pool_dict.pop('health_monitor', None)
-                    member_dicts = pool_dict.pop('members', [])
-                    sp_dict = pool_dict.pop('session_persistence', None)
-                    pool_dict['load_balancer_id'] = lb_dm.id
-                    del pool_dict['listener_id']
-                    pool_dm = self.pool.create(session, **pool_dict)
-                    if sp_dict:
-                        sp_dict['pool_id'] = pool_dm.id
-                        self.session_persistence.create(session, **sp_dict)
-                    if hm_dict:
-                        hm_dict['pool_id'] = pool_dm.id
-                        self.health_monitor.create(session, **hm_dict)
-                    for r_member_dict in member_dicts:
-                        r_member_dict['pool_id'] = pool_dm.id
-                        self.member.create(session, **r_member_dict)
-                    listener_dict['default_pool_id'] = pool_dm.id
-                self.listener.create(session, **listener_dict)
-                for sni_container in sni_containers:
-                    self.sni.create(session, **sni_container)
-                if l7policies_dict:
-                    for policy_dict in l7policies_dict:
-                        l7rules_dict = policy_dict.pop('l7rules')
-                        if policy_dict.get('redirect_pool'):
-                            r_pool_dict = policy_dict.pop(
-                                'redirect_pool')
-                            r_hm_dict = r_pool_dict.pop('health_monitor', None)
-                            r_sp_dict = r_pool_dict.pop('session_persistence',
-                                                        None)
-                            r_member_dicts = r_pool_dict.pop('members', [])
-                            if 'listener_id' in r_pool_dict.keys():
-                                del r_pool_dict['listener_id']
-                            r_pool_dm = self.pool.create(
-                                session, **r_pool_dict)
-                            if r_sp_dict:
-                                r_sp_dict['pool_id'] = r_pool_dm.id
-                                self.session_persistence.create(session,
-                                                                **r_sp_dict)
-                            if r_hm_dict:
-                                r_hm_dict['pool_id'] = r_pool_dm.id
-                                self.health_monitor.create(session,
-                                                           **r_hm_dict)
-                            for r_member_dict in r_member_dicts:
-                                r_member_dict['pool_id'] = r_pool_dm.id
-                                self.member.create(session, **r_member_dict)
-                            policy_dict['redirect_pool_id'] = r_pool_dm.id
-                        policy_dm = self.l7policy.create(session,
-                                                         **policy_dict)
-                        for rule_dict in l7rules_dict:
-                            rule_dict['l7policy_id'] = policy_dm.id
-                            self.l7rule.create(session, **rule_dict)
+            lock_session = db_api.get_session(autocommit=False)
+            try:
+                if self.check_quota_met(session,
+                                        lock_session,
+                                        data_models.LoadBalancer,
+                                        lb_dict['project_id']):
+                    raise exceptions.QuotaException
+                lb_dm = self.create_load_balancer_and_vip(
+                    lock_session, lb_dict, vip_dict)
+                for listener_dict in listener_dicts:
+                    # Add listener quota check
+                    if self.check_quota_met(session,
+                                            lock_session,
+                                            data_models.Listener,
+                                            lb_dict['project_id']):
+                        raise exceptions.QuotaException
+                    pool_dict = listener_dict.pop('default_pool', None)
+                    l7policies_dict = listener_dict.pop('l7policies', None)
+                    sni_containers = listener_dict.pop('sni_containers', [])
+                    if pool_dict:
+                        # Add pool quota check
+                        if self.check_quota_met(session,
+                                                lock_session,
+                                                data_models.Pool,
+                                                lb_dict['project_id']):
+                            raise exceptions.QuotaException
+                        hm_dict = pool_dict.pop('health_monitor', None)
+                        member_dicts = pool_dict.pop('members', [])
+                        sp_dict = pool_dict.pop('session_persistence', None)
+                        pool_dict['load_balancer_id'] = lb_dm.id
+                        del pool_dict['listener_id']
+                        pool_dm = self.pool.create(lock_session, **pool_dict)
+                        if sp_dict:
+                            sp_dict['pool_id'] = pool_dm.id
+                            self.session_persistence.create(lock_session,
+                                                            **sp_dict)
+                        if hm_dict:
+                            # Add hm quota check
+                            if self.check_quota_met(session,
+                                                    lock_session,
+                                                    data_models.HealthMonitor,
+                                                    lb_dict['project_id']):
+                                raise exceptions.QuotaException
+                            hm_dict['pool_id'] = pool_dm.id
+                            self.health_monitor.create(lock_session, **hm_dict)
+                        for r_member_dict in member_dicts:
+                            # Add member quota check
+                            if self.check_quota_met(session,
+                                                    lock_session,
+                                                    data_models.Member,
+                                                    lb_dict['project_id']):
+                                raise exceptions.QuotaException
+                            r_member_dict['pool_id'] = pool_dm.id
+                            self.member.create(lock_session, **r_member_dict)
+                        listener_dict['default_pool_id'] = pool_dm.id
+                    self.listener.create(lock_session, **listener_dict)
+                    for sni_container in sni_containers:
+                        self.sni.create(lock_session, **sni_container)
+                    if l7policies_dict:
+                        for policy_dict in l7policies_dict:
+                            l7rules_dict = policy_dict.pop('l7rules')
+                            if policy_dict.get('redirect_pool'):
+                                # Add pool quota check
+                                if self.check_quota_met(session,
+                                                        lock_session,
+                                                        data_models.Pool,
+                                                        lb_dict['project_id']):
+                                    raise exceptions.QuotaException
+                                r_pool_dict = policy_dict.pop(
+                                    'redirect_pool')
+                                r_hm_dict = r_pool_dict.pop('health_monitor',
+                                                            None)
+                                r_sp_dict = r_pool_dict.pop(
+                                    'session_persistence', None)
+                                r_member_dicts = r_pool_dict.pop('members', [])
+                                if 'listener_id' in r_pool_dict.keys():
+                                    del r_pool_dict['listener_id']
+                                r_pool_dm = self.pool.create(lock_session,
+                                                             **r_pool_dict)
+                                if r_sp_dict:
+                                    r_sp_dict['pool_id'] = r_pool_dm.id
+                                    self.session_persistence.create(
+                                        lock_session, **r_sp_dict)
+                                if r_hm_dict:
+                                    # Add hm quota check
+                                    if self.check_quota_met(
+                                            session,
+                                            lock_session,
+                                            data_models.HealthMonitor,
+                                            lb_dict['project_id']):
+                                        raise exceptions.QuotaException
+                                    r_hm_dict['pool_id'] = r_pool_dm.id
+                                    self.health_monitor.create(lock_session,
+                                                               **r_hm_dict)
+                                for r_member_dict in r_member_dicts:
+                                    # Add member quota check
+                                    if self.check_quota_met(
+                                            session,
+                                            lock_session,
+                                            data_models.Member,
+                                            lb_dict['project_id']):
+                                        raise exceptions.QuotaException
+                                    r_member_dict['pool_id'] = r_pool_dm.id
+                                    self.member.create(lock_session,
+                                                       **r_member_dict)
+                                policy_dict['redirect_pool_id'] = r_pool_dm.id
+                            policy_dm = self.l7policy.create(lock_session,
+                                                             **policy_dict)
+                            for rule_dict in l7rules_dict:
+                                rule_dict['l7policy_id'] = policy_dm.id
+                                self.l7rule.create(lock_session, **rule_dict)
+                lock_session.commit()
+            except Exception:
+                with excutils.save_and_reraise_exception():
+                    lock_session.rollback()
+
+        session.expire_all()
         return self.load_balancer.get(session, id=lb_dm.id)
 
 
@@ -322,12 +630,41 @@ class LoadBalancerRepository(BaseRepository):
         with session.begin(subtransactions=True):
             lb = session.query(self.model_class).with_for_update().filter_by(
                 id=id).one()
-            if lb.provisioning_status not in constants.MUTABLE_STATUSES:
+            is_delete = status == consts.PENDING_DELETE
+            acceptable_statuses = (
+                consts.DELETABLE_STATUSES
+                if is_delete else consts.MUTABLE_STATUSES
+            )
+            if lb.provisioning_status not in acceptable_statuses:
                 return False
             lb.provisioning_status = status
             session.add(lb)
             return True
 
+    def check_load_balancer_expired(self, session, lb_id, exp_age=None):
+        """Checks if a given load balancer is expired.
+
+        :param session: A Sql Alchemy database session.
+        :param lb_id: id of an load balancer object
+        :param exp_age: A standard datetime delta which is used to see for how
+        long can a load balancer live without updates before it is considered
+        expired (default: CONF.house_keeping.load_balancer_expiry_age)
+        :returns: boolean
+        """
+        if not exp_age:
+            exp_age = datetime.timedelta(
+                seconds=CONF.house_keeping.load_balancer_expiry_age)
+
+        timestamp = datetime.datetime.utcnow() - exp_age
+        lb = self.get(session, id=lb_id)
+        if lb:
+            # If a load balancer was never updated use its creation timestamp
+            last_update = lb.updated_at or lb.created_at
+            return last_update < timestamp
+        else:
+            # Load balancer was just deleted.
+            return True
+
 
 class VipRepository(BaseRepository):
     model_class = models.Vip
@@ -389,7 +726,7 @@ class ListenerRepository(BaseRepository):
                     listener.peer_port > max_peer_port):
                 max_peer_port = listener.peer_port
         if max_peer_port == 0:
-            return constants.HAPROXY_BASE_PEER_PORT
+            return consts.HAPROXY_BASE_PEER_PORT
         else:
             return max_peer_port + 1
 
@@ -451,17 +788,20 @@ class ListenerRepository(BaseRepository):
 class ListenerStatisticsRepository(BaseRepository):
     model_class = models.ListenerStatistics
 
-    def replace(self, session, listener_id, **model_kwargs):
+    def replace(self, session, listener_id, amphora_id, **model_kwargs):
         """replace or insert listener into database."""
         with session.begin(subtransactions=True):
             count = session.query(self.model_class).filter_by(
-                listener_id=listener_id).count()
+                listener_id=listener_id, amphora_id=amphora_id).count()
             if count:
                 session.query(self.model_class).filter_by(
-                    listener_id=listener_id).update(model_kwargs,
-                                                    synchronize_session=False)
+                    listener_id=listener_id,
+                    amphora_id=amphora_id).update(
+                    model_kwargs,
+                    synchronize_session=False)
             else:
                 model_kwargs['listener_id'] = listener_id
+                model_kwargs['amphora_id'] = amphora_id
                 self.create(session, **model_kwargs)
 
     def update(self, session, listener_id, **model_kwargs):
@@ -533,7 +873,7 @@ class AmphoraRepository(BaseRepository):
         """
         with session.begin(subtransactions=True):
             count = session.query(self.model_class).filter_by(
-                status=constants.AMPHORA_READY, load_balancer_id=None).count()
+                status=consts.AMPHORA_READY, load_balancer_id=None).count()
 
         return count
 
@@ -606,15 +946,16 @@ class AmphoraHealthRepository(BaseRepository):
 
         :param session: A Sql Alchemy database session.
         :param amphora_id: id of an amphora object
-        :param exp_age: A standard datetime which is used to see if an
-        amphora needs to be updated (default: now - 10s)
+        :param exp_age: A standard datetime delta which is used to see for how
+        long can an amphora live without updates before it is considered
+        expired (default: CONF.house_keeping.amphora_expiry_age)
         :returns: boolean
         """
         if not exp_age:
-            timestamp = datetime.datetime.utcnow() - datetime.timedelta(
-                seconds=10)
-        else:
-            timestamp = datetime.datetime.utcnow() - exp_age
+            exp_age = datetime.timedelta(
+                seconds=CONF.house_keeping.amphora_expiry_age)
+
+        timestamp = datetime.datetime.utcnow() - exp_age
         amphora_health = self.get(session, amphora_id=amphora_id)
         if amphora_health is not None:
             return amphora_health.last_update < timestamp
@@ -674,9 +1015,9 @@ class L7RuleRepository(BaseRepository):
                     l7rule_dict.update({k: v})
             # Clear out the 'key' attribute for rule types that don't use it.
             if ('type' in l7rule_dict.keys() and
-                l7rule_dict['type'] in (constants.L7RULE_TYPE_HOST_NAME,
-                                        constants.L7RULE_TYPE_PATH,
-                                        constants.L7RULE_TYPE_FILE_TYPE)):
+                l7rule_dict['type'] in (consts.L7RULE_TYPE_HOST_NAME,
+                                        consts.L7RULE_TYPE_PATH,
+                                        consts.L7RULE_TYPE_FILE_TYPE)):
                 l7rule_dict['key'] = None
                 model_kwargs.update({'key': None})
             validate.l7rule_data(self.model_class(**l7rule_dict))
@@ -712,7 +1053,7 @@ class L7PolicyRepository(BaseRepository):
 
     def _validate_l7policy_pool_data(self, session, l7policy):
         """Does validations on a given L7 policy."""
-        if l7policy.action == constants.L7POLICY_ACTION_REDIRECT_TO_POOL:
+        if l7policy.action == consts.L7POLICY_ACTION_REDIRECT_TO_POOL:
             session.expire(session.query(models.Listener).filter_by(
                 id=l7policy.listener_id).first())
             listener = (session.query(models.Listener).
@@ -747,14 +1088,14 @@ class L7PolicyRepository(BaseRepository):
 
             if l7policy.action:
                 model_kwargs.update(action=l7policy.action)
-                if l7policy.action == constants.L7POLICY_ACTION_REJECT:
+                if l7policy.action == consts.L7POLICY_ACTION_REJECT:
                     model_kwargs.update(redirect_url=None)
                     model_kwargs.update(redirect_pool_id=None)
                 elif (l7policy.action ==
-                        constants.L7POLICY_ACTION_REDIRECT_TO_URL):
+                        consts.L7POLICY_ACTION_REDIRECT_TO_URL):
                     model_kwargs.update(redirect_pool_id=None)
                 elif (l7policy.action ==
-                        constants.L7POLICY_ACTION_REDIRECT_TO_POOL):
+                        consts.L7POLICY_ACTION_REDIRECT_TO_POOL):
                     model_kwargs.update(redirect_url=None)
 
             l7policy_db.update(model_kwargs)
@@ -781,7 +1122,7 @@ class L7PolicyRepository(BaseRepository):
             # This is to work around unexpected / idiotic behavior of the
             # SQLAlchemy orderinglist extension.
             position = model_kwargs.pop('position', None)
-            model_kwargs.update(position=constants.MAX_POLICY_POSITION)
+            model_kwargs.update(position=consts.MAX_POLICY_POSITION)
             if not model_kwargs.get('id'):
                 model_kwargs.update(id=uuidutils.generate_uuid())
             if model_kwargs.get('redirect_pool_id'):
@@ -831,3 +1172,35 @@ class L7PolicyRepository(BaseRepository):
         session.refresh(listener)
         listener.l7policies.reorder()
         session.flush()
+
+
+class QuotasRepository(BaseRepository):
+    model_class = models.Quotas
+
+    def update(self, session, project_id, **model_kwargs):
+        with session.begin(subtransactions=True):
+            kwargs_quota = model_kwargs['quota']
+            quotas = session.query(self.model_class).filter_by(
+                project_id=project_id).with_for_update().first()
+            if not quotas:
+                quotas = models.Quotas(project_id=project_id)
+
+            for key, val in six.iteritems(kwargs_quota):
+                setattr(quotas, key, val)
+            session.add(quotas)
+            session.flush()
+        return self.get(session, project_id=project_id)
+
+    def delete(self, session, project_id):
+        with session.begin(subtransactions=True):
+            quotas = session.query(self.model_class).filter_by(
+                project_id=project_id).with_for_update().first()
+            if not quotas:
+                raise exceptions.NotFound(
+                    resource=data_models.Quotas._name(), id=project_id)
+            quotas.health_monitor = None
+            quotas.load_balancer = None
+            quotas.listener = None
+            quotas.member = None
+            quotas.pool = None
+            session.flush()
