@@ -3,16 +3,7 @@
 set -ex
 
 GATE_DEST=$BASE/new
-
-_DEVSTACK_LOCAL_CONFIG_TAIL=
-
-# Inject config from hook
-function load_conf_hook {
-    local hook="$1"
-    local GATE_HOOKS=$GATE_DEST/octavia/octavia/tests/contrib/hooks
-
-    _DEVSTACK_LOCAL_CONFIG_TAIL+=$'\n'"$(cat $GATE_HOOKS/$hook)"
-}
+DEVSTACK_PATH=$GATE_DEST/devstack
 
 export DEVSTACK_LOCAL_CONFIG+="
 enable_plugin barbican https://git.openstack.org/openstack/barbican
@@ -25,31 +16,86 @@ if egrep --quiet '(vmx|svm)' /proc/cpuinfo; then
     export DEVSTACK_GATE_LIBVIRT_TYPE=kvm
 fi
 
+function _setup_octavia_multinode {
+
+    PRIMARY_NODE_IP=$(cat /etc/nodepool/primary_node_private)
+    SUBNODE_IP=$(head -n1 /etc/nodepool/sub_nodes_private)
+
+    # OCTAVIA_CONTROLLER_IP_PORT_LIST are the ips inside the
+    # lb-mgmt network that the amphoras reach via heartbeats
+    COMMON_MULTINODE_CONFIG="
+        OCTAVIA_USE_PREGENERATED_CERTS=True
+        OCTAVIA_USE_PREGENERATED_SSH_KEY=True
+        OCTAVIA_CONTROLLER_IP_PORT_LIST=192.168.0.3:5555,192.168.0.4:5555
+
+        # Embedded fix for devstack bug/1629133 , this line can be removed once
+        # that bug is fixed
+        SUBNETPOOL_PREFIX_V4=10.0.0.0/16
+    "
+
+    export DEVSTACK_LOCAL_CONFIG+="$COMMON_MULTINODE_CONFIG
+        OCTAVIA_NODE=main
+        OCTAVIA_NODES=main:$PRIMARY_NODE_IP,second:$SUBNODE_IP
+        enable_service o-api-ha
+        OCTAVIA_MGMT_PORT_IP=192.168.0.3
+    "
+
+    export DEVSTACK_SUBNODE_CONFIG+="$COMMON_MULTINODE_CONFIG
+        OCTAVIA_NODE=second
+        enable_plugin octavia https://git.openstack.org/openstack/octavia
+        OCTAVIA_MGMT_PORT_IP=192.168.0.4
+    "
+}
 
 function _setup_octavia {
     export DEVSTACK_LOCAL_CONFIG+="
         enable_plugin octavia https://git.openstack.org/openstack/octavia
         "
+    # Use infra's cached version of the file
+    if [ -f /opt/stack/new/devstack/files/get-pip.py ]; then
+            export DEVSTACK_LOCAL_CONFIG+="
+        DIB_REPOLOCATION_pip_and_virtualenv=file:///opt/stack/new/devstack/files/get-pip.py
+        "
+    fi
     if [ "$testenv" != "apiv1" ]; then
         ENABLED_SERVICES+="octavia,o-cw,o-hk,o-hm,o-api,"
+        if [ "$DEVSTACK_GATE_TOPOLOGY" == "multinode" ]; then
+            _setup_octavia_multinode
+        fi
     fi
     if [ "$testenv" = "apiv1" ]; then
-        load_conf_hook apiv1
+       cat > "$DEVSTACK_PATH/local.conf" <<EOF
+[[post-config|/etc/octavia/octavia.conf]]
+[DEFAULT]
+debug = True
+
+[controller_worker]
+amphora_driver = amphora_noop_driver
+compute_driver = compute_noop_driver
+network_driver = network_noop_driver
+
+EOF
+
     fi
 
     if [ "$testenv" = "scenario" ]; then
-        load_conf_hook scenario
+       cat > "$DEVSTACK_PATH/local.conf" <<EOF
+[[post-config|/etc/octavia/octavia.conf]]
+[DEFAULT]
+debug = True
+
+EOF
+
     fi
 }
 
-# Make sure lbaasv2 is listed as enabled for tempest
-load_conf_hook api_extensions
 
 case "$testtype" in
 
     "dsvm-functional")
         PROJECT_NAME=octavia
         OCTAVIA_PATH=$GATE_DEST/$PROJECT_NAME
+        DEVSTACK_PATH=$GATE_DEST/devstack
         IS_GATE=True
         USE_CONSTRAINT_ENV=False
         export LOG_COLOR=False
@@ -88,7 +134,6 @@ case "$testtype" in
         fi
 
         export ENABLED_SERVICES
-        export DEVSTACK_LOCAL_CONFIG+=$'\n'"$_DEVSTACK_LOCAL_CONFIG_TAIL"
         "$GATE_DEST"/devstack-gate/devstack-vm-gate.sh
         ;;
 
