@@ -17,10 +17,13 @@ try:
     from http import cookiejar as cookielib
 except ImportError:
     import cookielib
+import os
 import shlex
+import shutil
 import socket
 import subprocess
 import tempfile
+import threading
 import time
 
 from oslo_log import log as logging
@@ -33,20 +36,24 @@ from tempest import config
 from tempest import exceptions
 from tempest.lib.common.utils import test_utils
 from tempest.lib import exceptions as lib_exc
+from tempest.scenario import manager
 from tempest import test
 
 
 from octavia.i18n import _
-from octavia.tests.tempest.common import manager
 from octavia.tests.tempest.v1.clients import health_monitors_client
 from octavia.tests.tempest.v1.clients import listeners_client
 from octavia.tests.tempest.v1.clients import load_balancers_client
 from octavia.tests.tempest.v1.clients import members_client
 from octavia.tests.tempest.v1.clients import pools_client
+from octavia.tests.tempest.v1.clients import quotas_client
 
 config = config.CONF
 
 LOG = logging.getLogger(__name__)
+HTTPD_SRC = os.path.abspath(
+    os.path.join(os.path.dirname(__file__),
+                 '../../../contrib/httpd.go'))
 
 
 class BaseTestCase(manager.NetworkScenarioTest):
@@ -59,10 +66,8 @@ class BaseTestCase(manager.NetworkScenarioTest):
         self.floating_ips = {}
         self.servers_floating_ips = {}
         self.server_ips = {}
-        self.port1 = 80
-        self.port2 = 88
+        self.start_port = 80
         self.num = 50
-        self.server_ips = {}
         self.server_fixed_ips = {}
 
         mgr = self.get_client_manager()
@@ -80,8 +85,8 @@ class BaseTestCase(manager.NetworkScenarioTest):
         self.health_monitors_client = (
             health_monitors_client.HealthMonitorsClient(
                 *self.client_args))
+        self.quotas_client = quotas_client.QuotasClient(*self.client_args)
 
-        self.tenant_id = self.load_balancers_client.tenant_id
         self._create_security_group_for_test()
         self._set_net_and_subnet()
 
@@ -111,15 +116,14 @@ class BaseTestCase(manager.NetworkScenarioTest):
         The configured private network and associated subnet is used as a
         fallback in absence of tenant networking.
         """
+        tenant_id = self.load_balancers_client.tenant_id
         try:
-            tenant_net = self.admin_manager.networks_client.list_networks(
-                tenant_id=self.tenant_id)['networks'][0]
+            tenant_net = self._list_networks(tenant_id=tenant_id)[0]
         except IndexError:
             tenant_net = None
 
         if tenant_net:
-            tenant_subnet = self.admin_manager.subnets_client.list_subnets(
-                tenant_id=self.tenant_id)['subnets'][0]
+            tenant_subnet = self._list_subnets(tenant_id=tenant_id)[0]
             self.subnet = tenant_subnet
             self.network = tenant_net
         else:
@@ -129,15 +133,14 @@ class BaseTestCase(manager.NetworkScenarioTest):
             # with the fixed network is the one we want.  In the future, we
             # should instead pull a subnet id from config, which is set by
             # devstack/admin/etc.
-            subnet = self.admin_manager.subnets_client.list_subnets(
-                network_id=self.network['id'])['subnets'][0]
+            subnet = self._list_subnets(network_id=self.network['id'])[0]
             self.subnet = subnet
 
     def _create_security_group_for_test(self):
         self.security_group = self._create_security_group(
-            tenant_id=self.tenant_id)
-        self._create_security_group_rules_for_port(self.port1)
-        self._create_security_group_rules_for_port(self.port2)
+            tenant_id=self.load_balancers_client.tenant_id)
+        self._create_security_group_rules_for_port(self.start_port)
+        self._create_security_group_rules_for_port(self.start_port + 1)
 
     def _create_security_group_rules_for_port(self, port):
         rule = {
@@ -148,12 +151,13 @@ class BaseTestCase(manager.NetworkScenarioTest):
         }
         self._create_security_group_rule(
             secgroup=self.security_group,
-            tenant_id=self.tenant_id,
+            tenant_id=self.load_balancers_client.tenant_id,
             **rule)
 
     def _ipv6_subnet(self, address6_mode):
-        router = self._get_router(tenant_id=self.tenant_id)
-        self.network = self._create_network(tenant_id=self.tenant_id)
+        tenant_id = self.load_balancers_client.tenant_id
+        router = self._get_router(tenant_id=tenant_id)
+        self.network = self._create_network(tenant_id=tenant_id)
         self.subnet = self._create_subnet(network=self.network,
                                           namestart='sub6',
                                           ip_version=6,
@@ -198,88 +202,77 @@ class BaseTestCase(manager.NetworkScenarioTest):
         self.server_fixed_ips[server['id']] = (
             server['addresses'][net_name][0]['addr'])
         self.assertTrue(self.servers_keypairs)
+        self.servers[name] = server['id']
         return server
 
-    def _create_servers(self):
-        for count in range(2):
-            self.server = self._create_server(name=("server%s" % (count + 1)))
-            if count == 0:
-                self.servers['primary'] = self.server['id']
-            else:
-                self.servers['secondary'] = self.server['id']
-        self.assertEqual(len(self.servers_keypairs), 2)
+    def _create_servers(self, num=2):
+        for count in range(num):
+            name = "server%s" % (count + 1)
+            self.server = self._create_server(name=name)
+        self.assertEqual(len(self.servers_keypairs), num)
 
-    def _stop_server(self):
-        for name, value in six.iteritems(self.servers):
-            if name == 'primary':
-                LOG.info(('STOPPING SERVER: {0}'.format(name)))
+    def _stop_server(self, name):
+        for sname, value in six.iteritems(self.servers):
+            if sname == name:
+                LOG.info(('STOPPING SERVER: {0}'.format(sname)))
                 self.servers_client.stop_server(value)
                 waiters.wait_for_server_status(self.servers_client,
                                                value, 'SHUTOFF')
         LOG.info(('STOPPING SERVER COMPLETED!'))
 
-    def _start_server(self):
-        for name, value in six.iteritems(self.servers):
-            if name == 'primary':
+    def _start_server(self, name):
+        for sname, value in six.iteritems(self.servers):
+            if sname == name:
                 self.servers_client.start(value)
                 waiters.wait_for_server_status(self.servers_client,
                                                value, 'ACTIVE')
 
-    def _start_servers(self):
-        """Start one or more backends
+    def _build_static_httpd(self):
+        """Compile test httpd as a static binary
 
-        1. SSH to the instance
-        2. Start two http backends listening on ports 80 and 88 respectively
+        returns file path of resulting binary file
         """
+        builddir = tempfile.mkdtemp()
+        shutil.copyfile(HTTPD_SRC, os.path.join(builddir, 'httpd.go'))
+        self.execute('go build -ldflags '
+                     '"-linkmode external -extldflags -static" '
+                     'httpd.go', cwd=builddir)
+        return os.path.join(builddir, 'httpd')
+
+    def _start_backend_httpd_processes(self, backend, ports=None):
+        """Start one or more webservers on a given backend server
+
+        1. SSH to the backend
+        2. Start http backends listening on the given ports
+        """
+        ports = ports or [80, 81]
+        httpd = self._build_static_httpd()
+        backend_id = self.servers[backend]
         for server_id, ip in six.iteritems(self.server_ips):
+            if server_id != backend_id:
+                continue
             private_key = self.servers_keypairs[server_id]['private_key']
-            server = self.servers_client.show_server(server_id)['server']
-            server_name = server['name']
             username = config.validation.image_ssh_user
             ssh_client = self.get_remote_client(
                 ip_address=ip,
                 private_key=private_key)
 
-            # Write a backend's response into a file
-            resp = ('echo -ne "HTTP/1.1 200 OK\r\nContent-Length: 7\r\n'
-                    'Set-Cookie:JSESSIONID=%(s_id)s\r\nConnection: close\r\n'
-                    'Content-Type: text/html; '
-                    'charset=UTF-8\r\n\r\n%(server)s"; cat >/dev/null')
+            with tempfile.NamedTemporaryFile() as key:
+                key.write(private_key)
+                key.flush()
+                self.copy_file_to_host(httpd,
+                                       "/dev/shm/httpd",
+                                       ip,
+                                       username, key.name)
 
-            with tempfile.NamedTemporaryFile() as script:
-                script.write(resp % {'s_id': server_name[-1],
-                                     'server': server_name})
-                script.flush()
-                with tempfile.NamedTemporaryFile() as key:
-                    key.write(private_key)
-                    key.flush()
-                    self.copy_file_to_host(script.name,
-                                           "/tmp/script1",
-                                           ip,
-                                           username, key.name)
-
-            # Start netcat
-            start_server = ('while true; do '
-                            'sudo nc -ll -p %(port)s -e sh /tmp/%(script)s; '
-                            'done > /dev/null &')
-            cmd = start_server % {'port': self.port1,
-                                  'script': 'script1'}
-            ssh_client.exec_command(cmd)
-
-            if len(self.server_ips) == 1:
-                with tempfile.NamedTemporaryFile() as script:
-                    script.write(resp % {'s_id': 2,
-                                         'server': 'server2'})
-                    script.flush()
-                    with tempfile.NamedTemporaryFile() as key:
-                        key.write(private_key)
-                        key.flush()
-                        self.copy_file_to_host(script.name,
-                                               "/tmp/script2", ip,
-                                               username, key.name)
-                cmd = start_server % {'port': self.port2,
-                                      'script': 'script2'}
+            # Start httpd
+            start_server = ('sudo sh -c "ulimit -n 100000; screen -d -m '
+                            '/dev/shm/httpd -id %(id)s -port %(port)s"')
+            for i in range(len(ports)):
+                cmd = start_server % {'id': backend + "_" + str(i),
+                                      'port': ports[i]}
                 ssh_client.exec_command(cmd)
+            # Allow ssh_client connection to fall out of scope
 
     def _create_listener(self, load_balancer_id, default_pool_id=None):
         """Create a listener with HTTP protocol listening on port 80."""
@@ -290,8 +283,12 @@ class BaseTestCase(manager.NetworkScenarioTest):
             lb_id=load_balancer_id,
             **self.create_listener_kwargs)
         self.assertTrue(self.listener)
-        self.addCleanup(self._cleanup_listener, load_balancer_id,
-                        self.listener.get('id'))
+        self.addCleanup(self._cleanup_listener, self.listener['id'],
+                        load_balancer_id)
+        LOG.info(('Waiting for lb status on create listener id: {0}'.format(
+            self.listener['id'])))
+        self._wait_for_load_balancer_status(load_balancer_id)
+
         return self.listener
 
     def _create_health_monitor(self):
@@ -303,8 +300,8 @@ class BaseTestCase(manager.NetworkScenarioTest):
             pool_id=self.pool['id'])
         self.assertTrue(self.hm)
         self.addCleanup(self._cleanup_health_monitor,
-                        load_balancer_id=self.load_balancer['id'],
-                        pool_id=self.pool['id'])
+                        pool_id=self.pool['id'],
+                        load_balancer_id=self.load_balancer['id'])
         self._wait_for_load_balancer_status(self.load_balancer['id'])
 
         # add clean up members prior to clean up of health monitor
@@ -334,8 +331,10 @@ class BaseTestCase(manager.NetworkScenarioTest):
         self.pool = self.pools_client.create_pool(lb_id=load_balancer_id,
                                                   **create_pool_kwargs)
         self.assertTrue(self.pool)
-        self.addCleanup(self._cleanup_pool, load_balancer_id,
-                        self.pool['id'])
+        self.addCleanup(self._cleanup_pool, self.pool['id'], load_balancer_id)
+        LOG.info(('Waiting for lb status on create pool id: {0}'.format(
+            self.pool['id'])))
+        self._wait_for_load_balancer_status(load_balancer_id)
         return self.pool
 
     def _cleanup_load_balancer(self, load_balancer_id):
@@ -362,41 +361,22 @@ class BaseTestCase(manager.NetworkScenarioTest):
         if load_balancer_id:
             self._wait_for_load_balancer_status(load_balancer_id, delete=True)
 
-    def _create_members(self, load_balancer_id, pool_id, subnet_id=None):
-        """Create one or more Members
+    def _create_members(self, load_balancer_id, pool_id, backend, ports=None,
+                        subnet_id=None):
+        """Create one or more Members based on the given backend
 
-        In case there is only one server, create both members with the same ip
-        but with different ports to listen on.
+        :backend: The backend server the members will be on
+        :param ports: List of listening ports on the backend server
         """
+        ports = ports or [80, 81]
+        backend_id = self.servers[backend]
         for server_id, ip in six.iteritems(self.server_fixed_ips):
-            if len(self.server_fixed_ips) == 1:
+            if server_id != backend_id:
+                continue
+            for port in ports:
                 create_member_kwargs = {
                     'ip_address': ip,
-                    'protocol_port': self.port1,
-                    'weight': 50,
-                    'subnet_id': subnet_id
-                }
-                member1 = self.members_client.create_member(
-                    lb_id=load_balancer_id,
-                    pool_id=pool_id,
-                    **create_member_kwargs)
-                self._wait_for_load_balancer_status(load_balancer_id)
-                create_member_kwargs = {
-                    'ip_address': ip,
-                    'protocol_port': self.port2,
-                    'weight': 50,
-                    'subnet_id': subnet_id
-                }
-                member2 = self.members_client.create_member(
-                    lb_id=load_balancer_id,
-                    pool_id=pool_id,
-                    **create_member_kwargs)
-                self._wait_for_load_balancer_status(load_balancer_id)
-                self.members.extend([member1, member2])
-            else:
-                create_member_kwargs = {
-                    'ip_address': ip,
-                    'protocol_port': self.port1,
+                    'protocol_port': port,
                     'weight': 50,
                     'subnet_id': subnet_id
                 }
@@ -404,9 +384,10 @@ class BaseTestCase(manager.NetworkScenarioTest):
                     lb_id=load_balancer_id,
                     pool_id=pool_id,
                     **create_member_kwargs)
+                LOG.info(('Waiting for lb status on create member...'))
                 self._wait_for_load_balancer_status(load_balancer_id)
                 self.members.append(member)
-        self.assertTrue(self.members)
+            self.assertTrue(self.members)
 
     def _assign_floating_ip_to_lb_vip(self, lb):
         public_network_id = config.network.public_network_id
@@ -460,37 +441,26 @@ class BaseTestCase(manager.NetworkScenarioTest):
                  .format(fp=floating_ip, st=status))
 
     def _create_load_balancer(self, ip_version=4, persistence_type=None):
-        self.create_lb_kwargs = {'vip': {'subnet_id': self.subnet['id']}}
+        """Create a load balancer.
+
+        Also assigns a floating IP to the created load balancer.
+
+        :param ip_version: IP version to be used for the VIP IP
+        :returns: ID of the created load balancer
+        """
+        self.create_lb_kwargs = {
+            'vip': {'subnet_id': self.subnet['id']},
+            'project_id': self.load_balancers_client.tenant_id}
         self.load_balancer = self.load_balancers_client.create_load_balancer(
             **self.create_lb_kwargs)
-        load_balancer_id = self.load_balancer['id']
-        self.addCleanup(self._cleanup_load_balancer, load_balancer_id)
+        lb_id = self.load_balancer['id']
+        self.addCleanup(self._cleanup_load_balancer, lb_id)
         LOG.info(('Waiting for lb status on create load balancer id: {0}'
-                  .format(load_balancer_id)))
-        self._wait_for_load_balancer_status(load_balancer_id=load_balancer_id,
-                                            provisioning_status='ACTIVE',
-                                            operating_status='ONLINE')
-
-        self.pool = self._create_pool(load_balancer_id=load_balancer_id,
-                                      persistence_type=persistence_type)
-        self._wait_for_load_balancer_status(load_balancer_id)
-        LOG.info(('Waiting for lb status on create pool id: {0}'.format(
-            self.pool['id'])))
-
-        self.listener = self._create_listener(
-            load_balancer_id=load_balancer_id,
-            default_pool_id=self.pool['id']
-        )
-        self._wait_for_load_balancer_status(load_balancer_id)
-        LOG.info(('Waiting for lb status on create listener id: {0}'.format(
-            self.listener['id'])))
-
-        self._create_members(load_balancer_id=load_balancer_id,
-                             pool_id=self.pool['id'],
-                             subnet_id=self.subnet['id'])
+                  .format(lb_id)))
         self.load_balancer = self._wait_for_load_balancer_status(
-            load_balancer_id)
-        LOG.info(('Waiting for lb status on create members...'))
+            load_balancer_id=lb_id,
+            provisioning_status='ACTIVE',
+            operating_status='ONLINE')
 
         self.vip_ip = self.load_balancer['vip'].get('ip_address')
 
@@ -511,13 +481,44 @@ class BaseTestCase(manager.NetworkScenarioTest):
         self.ports_client_admin.update_port(
             self.load_balancer['vip']['port_id'],
             security_groups=[self.security_group['id']])
+        return lb_id
+
+    def _create_load_balancer_over_quota(self):
+        """Attempt to create a load balancer over quota.
+
+        Creates two load balancers one after the other expecting
+        the second create to exceed the configured quota.
+
+        :returns: Response body from the request
+        """
+        self.create_lb_kwargs = {
+            'vip': {'subnet_id': self.subnet['id']},
+            'project_id': self.load_balancers_client.tenant_id}
+        self.load_balancer = self.load_balancers_client.create_load_balancer(
+            **self.create_lb_kwargs)
+        lb_id = self.load_balancer['id']
+        self.addCleanup(self._cleanup_load_balancer, lb_id)
+
+        self.create_lb_kwargs = {
+            'vip': {'subnet_id': self.subnet['id']},
+            'project_id': self.load_balancers_client.tenant_id}
+        lb_client = self.load_balancers_client
+        lb_client.create_load_balancer_over_quota(
+            **self.create_lb_kwargs)
+
+        LOG.info(('Waiting for lb status on create load balancer id: {0}'
+                  .format(lb_id)))
+        self.load_balancer = self._wait_for_load_balancer_status(
+            load_balancer_id=lb_id,
+            provisioning_status='ACTIVE',
+            operating_status='ONLINE')
 
     def _wait_for_load_balancer_status(self, load_balancer_id,
                                        provisioning_status='ACTIVE',
                                        operating_status='ONLINE',
                                        delete=False):
-        interval_time = 1
-        timeout = 600
+        interval_time = config.octavia.lb_build_interval
+        timeout = config.octavia.lb_build_timeout
         end_time = time.time() + timeout
         while time.time() < end_time:
             try:
@@ -563,8 +564,8 @@ class BaseTestCase(manager.NetworkScenarioTest):
         return lb
 
     def _wait_for_pool_session_persistence(self, pool_id, sp_type=None):
-        interval_time = 1
-        timeout = 10
+        interval_time = config.octavia.build_interval
+        timeout = config.octavia.build_timeout
         end_time = time.time() + timeout
         while time.time() < end_time:
             pool = self.pools_client.get_pool(self.load_balancer['id'],
@@ -582,23 +583,36 @@ class BaseTestCase(manager.NetworkScenarioTest):
                   pool_id=pool_id,
                   type=sp_type))
 
-    def _check_load_balancing(self):
-        """Check Load Balancing
+    def _check_members_balanced(self, members=None):
+        """Check that back-end members are load balanced.
 
-        1. Send NUM requests on the floating ip associated with the VIP
-        2. Check that the requests are shared between the two servers
+        1. Send requests on the floating ip associated with the VIP
+        2. Check that the requests are shared between the members given
+        3. Check that no unexpected members were balanced.
         """
-        LOG.info(_('Checking load balancing...'))
-        self._check_connection(self.vip_ip)
+        members = members or ['server1_0', 'server1_1']
+        LOG.info(_('Checking all members are balanced...'))
+        self._wait_for_http_service(self.vip_ip)
         LOG.info(_('Connection to {vip} is valid').format(vip=self.vip_ip))
-        counters = self._send_requests(self.vip_ip,
-                                       ["server1", "server2"])
+        counters = self._send_concurrent_requests(self.vip_ip)
         for member, counter in six.iteritems(counters):
+            LOG.info(_('Member {member} saw {counter} requests.').format(
+                member=member, counter=counter))
             self.assertGreater(counter, 0,
                                'Member %s never balanced' % member)
-        LOG.info(_('Done checking load balancing...'))
+        for member in members:
+            if member not in list(counters):
+                raise Exception(
+                    _("Member {member} was never balanced.").format(
+                        member=member))
+        for member in list(counters):
+            if member not in members:
+                raise Exception(
+                    _("Member {member} was balanced when it should not "
+                      "have been.").format(member=member))
+        LOG.info(_('Done checking all members are balanced...'))
 
-    def _check_connection(self, check_ip, port=80):
+    def _wait_for_http_service(self, check_ip, port=80):
         def try_connect(check_ip, port):
             try:
                 LOG.info(('checking connection to ip: {0} port: {1}'.format(
@@ -621,26 +635,67 @@ class BaseTestCase(manager.NetworkScenarioTest):
             if (time.time() - start) > timeout:
                 message = "Timed out trying to connect to %s" % check_ip
                 raise exceptions.TimeoutException(message)
+            time.sleep(1)
 
-    def _send_requests(self, vip_ip, servers):
-        counters = dict.fromkeys(servers, 0)
+    def _send_requests(self, vip_ip, path=''):
+        counters = dict()
         for i in range(self.num):
             try:
-                server = urllib2.urlopen("http://{0}/".format(vip_ip),
+                server = urllib2.urlopen("http://{0}/{1}".format(vip_ip, path),
                                          None, 2).read()
-                counters[server] += 1
+                if server not in counters:
+                    counters[server] = 1
+                else:
+                    counters[server] += 1
             # HTTP exception means fail of server, so don't increase counter
             # of success and continue connection tries
             except (error.HTTPError, error.URLError, socket.timeout) as e:
                 LOG.info(('Got Error in sending request: {0}'.format(e)))
                 continue
-        # Check that we had at least sent more than 1 request
-        counted = 0
-        for server, counter in counters.items():
-            counted += counter
-        self.assertGreater(counted, 1)
         return counters
 
+    def _send_concurrent_requests(self, vip_ip, path='', clients=5,
+                                  timeout=None):
+        class ClientThread(threading.Thread):
+            def __init__(self, test_case, cid, vip_ip, path=''):
+                super(ClientThread, self).__init__(
+                    name='ClientThread-{0}'.format(cid))
+                self.vip_ip = vip_ip
+                self.path = path
+                self.test_case = test_case
+                self.counters = dict()
+
+            def run(self):
+                # NOTE(dlundquist): _send_requests() does not mutate
+                # BaseTestCase so concurrent uses of _send_requests does not
+                # require a mutex.
+                self.counters = self.test_case._send_requests(self.vip_ip,
+                                                              path=self.path)
+
+            def join(self, timeout=None):
+                start = time.time()
+                super(ClientThread, self).join(timeout)
+                return time.time() - start
+
+        client_threads = [ClientThread(self, i, vip_ip, path=path)
+                          for i in range(clients)]
+        for ct in client_threads:
+            ct.start()
+        if timeout is None:
+            # timeout for all client threads defaults to 400ms per request
+            timeout = self.num * 0.4
+        total_counters = dict()
+        for ct in client_threads:
+            timeout -= ct.join(timeout)
+            if timeout <= 0:
+                LOG.error("Client thread {0} timed out".format(ct.name))
+                return dict()
+            for server in list(ct.counters):
+                if server not in total_counters:
+                    total_counters[server] = 0
+                total_counters[server] += ct.counters[server]
+        return total_counters
+
     def _traffic_validation_after_stopping_server(self):
         """Check that the requests are sent to the only ACTIVE server."""
 
@@ -657,12 +712,14 @@ class BaseTestCase(manager.NetworkScenarioTest):
     def _check_load_balancing_after_deleting_resources(self):
         """Check load balancer after deleting resources
 
-        Check that the requests are not sent to any servers
-        Assert that no traffic is sent to any servers
+        Assert that no traffic is sent to any backend servers
         """
-        counters = self._send_requests(self.vip_ip, ["server1", "server2"])
-        for member, counter in six.iteritems(counters):
-            self.assertEqual(counter, 0, 'Member %s is balanced' % member)
+        counters = self._send_requests(self.vip_ip)
+        if counters:
+            for server, counter in six.iteritems(counters):
+                self.assertEqual(
+                    counter, 0,
+                    'Server %s saw requests when it should not have' % server)
 
     def _check_source_ip_persistence(self):
         """Check source ip session persistence.
@@ -671,7 +728,7 @@ class BaseTestCase(manager.NetworkScenarioTest):
         that handled it the first time.
         """
         # Check that backends are reachable
-        self._check_connection(self.vip_ip)
+        self._wait_for_http_service(self.vip_ip)
 
         resp = []
         for count in range(10):
@@ -758,9 +815,13 @@ class BaseTestCase(manager.NetworkScenarioTest):
                "-i %(pkey)s %(file1)s %(dest)s" % {'pkey': pkey,
                                                    'file1': file_from,
                                                    'dest': dest})
+        return self.execute(cmd)
+
+    def execute(self, cmd, cwd=None):
         args = shlex.split(cmd.encode('utf-8'))
         subprocess_args = {'stdout': subprocess.PIPE,
-                           'stderr': subprocess.STDOUT}
+                           'stderr': subprocess.STDOUT,
+                           'cwd': cwd}
         proc = subprocess.Popen(args, **subprocess_args)
         stdout, stderr = proc.communicate()
         if proc.returncode != 0:
@@ -768,3 +829,76 @@ class BaseTestCase(manager.NetworkScenarioTest):
                       "output {2}, error {3}").format(cmd, proc.returncode,
                                                       stdout, stderr))
         return stdout
+
+    def _set_quotas(self, project_id=None, load_balancer=20, listener=20,
+                    pool=20, health_monitor=20, member=20):
+        if not project_id:
+            project_id = self.networks_client.tenant_id
+        body = {'quota': {
+            'load_balancer': load_balancer, 'listener': listener,
+            'pool': pool, 'health_monitor': health_monitor, 'member': member}}
+        return self.quotas_client.update_quotas(project_id, **body)
+
+    def _create_load_balancer_tree(self, ip_version=4):
+        # TODO(ptoohill): remove or null out project ID when Octavia supports
+        # keystone auth and automatically populates it for us.
+        project_id = self.networks_client.tenant_id
+
+        create_members = self._create_members_kwargs(self.subnet['id'])
+        create_pool = {'project_id': project_id,
+                       'lb_algorithm': 'ROUND_ROBIN',
+                       'protocol': 'HTTP',
+                       'members': create_members}
+        create_listener = {'project_id': project_id,
+                           'protocol': 'HTTP',
+                           'protocol_port': 80,
+                           'default_pool': create_pool}
+        create_lb = {'project_id': project_id,
+                     'vip': {'subnet_id': self.subnet['id']},
+                     'listeners': [create_listener]}
+
+        # Set quotas back and finish the test
+        self._set_quotas(project_id=project_id)
+        self.load_balancer = (self.load_balancers_client
+                              .create_load_balancer_graph(create_lb))
+
+        load_balancer_id = self.load_balancer['id']
+        self.addCleanup(self._cleanup_load_balancer, load_balancer_id)
+        LOG.info(('Waiting for lb status on create load balancer id: {0}'
+                  .format(load_balancer_id)))
+        self.load_balancer = self._wait_for_load_balancer_status(
+            load_balancer_id)
+
+        self.vip_ip = self.load_balancer['vip'].get('ip_address')
+
+        # if the ipv4 is used for lb, then fetch the right values from
+        # tempest.conf file
+        if ip_version == 4:
+            if (config.network.public_network_id and
+                    not config.network.project_networks_reachable):
+                load_balancer = self.load_balancer
+                self._assign_floating_ip_to_lb_vip(load_balancer)
+                self.vip_ip = self.floating_ips[
+                    load_balancer['id']][0]['floating_ip_address']
+
+        # Currently the ovs-agent is not enforcing security groups on the
+        # vip port - see https://bugs.launchpad.net/neutron/+bug/1163569
+        # However the linuxbridge-agent does, and it is necessary to add a
+        # security group with a rule that allows tcp port 80 to the vip port.
+        self.ports_client_admin.update_port(
+            self.load_balancer['vip']['port_id'],
+            security_groups=[self.security_group['id']])
+
+    def _create_members_kwargs(self, subnet_id=None):
+        """Create one or more Members
+
+        In case there is only one server, create both members with the same ip
+        but with different ports to listen on.
+        """
+        create_member_kwargs = []
+        for server_id, ip in six.iteritems(self.server_fixed_ips):
+            create_member_kwargs.append({'ip_address': ip,
+                                         'protocol_port': 80,
+                                         'weight': 50,
+                                         'subnet_id': subnet_id})
+        return create_member_kwargs
